@@ -22,8 +22,8 @@
 /* 
 	TTBD:
 
-		Would be nice to have an option of providing two .ti3 files
-		instead of doing the measurements here.
+		Would be nice to have an option of providing two ICC profiles,
+		instead of using .ti3 files (?? How well would it work though ?)
 
  */
 
@@ -48,13 +48,13 @@
 #include "dispwin.h"
 #include "dispsup.h"
 #include "ccmx.h"
-
-#include "spyd2setup.h"			/* Enable Spyder 2 access */
+#include "spyd2setup.h"
 
 #if defined (NT)
 #include <conio.h>
 #endif
 
+#define DEFAULT_MSTEPS 3
 #undef SHOW_WINDOW_ONFAKE	/* Display a test window up for a fake device */
 #define COMPORT 1			/* Default com port 1..4 */
 #define VERBOUT stdout
@@ -79,7 +79,7 @@ usage(char *diag, ...) {
 	disppath **dp;
 
 	fprintf(stderr,"Create Colorimeter Correction Matrix, Version %s\n",ARGYLL_VERSION_STR);
-	fprintf(stderr,"Author: Graeme W. Gill, licensed under the GPL Version 3\n");
+	fprintf(stderr,"Author: Graeme W. Gill, licensed under the AGPL Version 3\n");
 	if (setup_spyd2() == 2)
 		fprintf(stderr,"WARNING: This file contains a proprietary firmware image, and may not be freely distributed !\n");
 	if (diag != NULL) {
@@ -90,7 +90,7 @@ usage(char *diag, ...) {
 		va_end(args);
 		fprintf(stderr,"\n");
 	}
-	fprintf(stderr,"usage: ccmxmake [-options] output.sp\n");
+	fprintf(stderr,"usage: ccmxmake [-options] output.ccmx\n");
 	fprintf(stderr," -v                     Verbose mode\n");
 	fprintf(stderr," -f file1.ti3,file2.ti3 Create from two .ti3 files rather than measure.\n");
 #if defined(UNIX) && !defined(__APPLE__)
@@ -129,6 +129,7 @@ usage(char *diag, ...) {
 	fprintf(stderr," -C \"command\"           Invoke shell \"command\" each time a color is set\n");
 	fprintf(stderr," -o observ              Choose CIE Observer for spectrometer data:\n");
 	fprintf(stderr,"                        1931_2 (def), 1964_10, S&B 1955_2, shaw, J&V 1978_2\n");
+	fprintf(stderr," -s steps               Override default patch sequence combination steps  (default %d)\n",DEFAULT_MSTEPS);
 	fprintf(stderr," -W n|h|x               Override serial port flow control: n = none, h = HW, x = Xon/Xoff\n");
 	fprintf(stderr," -D [level]             Print debug diagnostics to stderr\n");
 	fprintf(stderr," -I displayname         Set display make and model description\n");
@@ -163,9 +164,9 @@ int main(int argc, char *argv[])
 	int adaptive = 0;					/* Use adaptive mode if available */
 	int dtype = 0;						/* Display kind, 0 = default, 1 = CRT, 2 = LCD */
 	int proj = 0;						/* NZ if projector */
-	int nocal = 0;						/* Disable auto calibration */
+	int noautocal = 0;					/* Disable auto calibration */
 	char *ccallout = NULL;				/* Change color Shell callout */
-	int msteps = 3;						/* Patch surface size */
+	int msteps = DEFAULT_MSTEPS;						/* Patch surface size */
 	int npat = 0;						/* Number of patches/colors */
 	ary3 *refs = NULL;					/* Reference XYZ values */
 	int gotref = 0;
@@ -315,7 +316,7 @@ int main(int argc, char *argv[])
 
 			/* No auto calibration */
 			} else if (argv[fa][1] == 'N') {
-				nocal = 1;
+				noautocal = 1;
 
 			/* High res spectral mode */
 			} else if (argv[fa][1] == 'H') {
@@ -347,6 +348,13 @@ int main(int argc, char *argv[])
 				} else
 					usage("-o parameter '%s' not recognised",na);
 
+			} else if (argv[fa][1] == 's') {
+				fa = nfa;
+				if (na == NULL) usage("Parameter expecte after -s");
+				msteps = atoi(na);
+				if (msteps < 2 || msteps > 16)
+					usage("-s parameter value %d is outside the range 2 to 16",msteps);
+
 			/* Change color callout */
 			} else if (argv[fa][1] == 'C') {
 				fa = nfa;
@@ -719,6 +727,8 @@ int main(int argc, char *argv[])
 				if (j >= 3)
 					break;		/* Done grid */
 			}
+			if (verb)
+				printf("Total test patches = %d\n",npat);
 		}
 
 		/* Until the measurements are done, or we give up */
@@ -830,7 +840,7 @@ int main(int argc, char *argv[])
 				inst2_capability cap2 = inst2_unknown;	/* Instrument capabilities 2 */
 
 				if ((dr = new_disprd(&errc, itype, fake ? -99 : comno, fc, dtype, proj, adaptive,
-				                     nocal, highres, 1, NULL, 0, disp, blackbg, override,
+				                     noautocal, highres, 1, NULL, 0, 0, disp, blackbg, override,
 				                     ccallout, NULL, patsize, ho, vo, NULL, spec, icxOT_none,
 				                     0, 0, verb, VERBOUT, debug, "fake" ICC_FILE_EXT)) == NULL)
 					error("new_disprd failed with '%s'\n",disprd_err(errc));
