@@ -14,8 +14,8 @@
  * Copyright 1996 - 2008 Graeme W. Gill
  * All rights reserved.
  *
- * This material is licenced under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 :-
- * see the License.txt file for licencing details.
+ * This material is licenced under the GNU GENERAL PUBLIC LICENSE Version 2 or later :-
+ * see the License2.txt file for licencing details.
  * 
  * Derived from icoms.h
  */
@@ -114,7 +114,6 @@ int set_normal_priority();
 struct _athread {
 #if defined (NT)
 	HANDLE th;				/* Thread */
-	DWORD thid;				/* Thread ID */
 #endif
 #if defined (UNIX) || defined(__APPLE__)
 	pthread_t thid;			/* Thread ID */
@@ -139,20 +138,6 @@ struct _athread {
 /* It should return 0 on completion or exit, nz on error. */
 athread *new_athread(int (*function)(void *context), void *context);
 
-#ifdef NEVER
-
-/* Ideas for worker variant on thread: */
-
-	/* Create a new worker thread, and put it to sleep */
-	athread *new_aworker();
-	/* Give the worker a job to do */
-   	         ->start_work(int (*function)(void *context), void *context);
-	/* See if the worker is finished its job */
-	result = ->poll_work();
-	/* Wait until the worker has finished its job */
-	result = ->wait_work();
-
-#endif
 
 /* - - - - - - - - - - - - - - - - - - -- */
 
@@ -165,18 +150,60 @@ int create_parent_directories(char *path);
 
 /* - - - - - - - - - - - - - - - - - - -- */
 
+struct _kkill_nproc_ctx {
+	athread *th;
+	char **pname;
+	int debug;
+	int stop;
+	int done;
+    void (*del)(struct _kkill_nproc_ctx *p);
+}; typedef struct _kkill_nproc_ctx kkill_nproc_ctx;
+
 #ifdef __APPLE__
 
-/* Kill a particular named process. */
+/* Kill a list of named processes. NULL for last */
 /* return < 0 if this fails. */
 /* return 0 if there is no such process */
 /* return 1 if a process was killed */
-int kill_nprocess(char *pname);
+int kill_nprocess(char **pname, int debug);
+
+/* Start a thread to constantly kill a process. */
+/* Call ctx->del() when done */
+kkill_nproc_ctx *kkill_nprocess(char **pname, int debug);
 
 #endif /* __APPLE__ */
 
 #include "xdg_bds.h"
 
+/* - - - - - - - - - - - - - - - - - - -- */
+/* A very small subset of icclib */
+#ifdef SALONEINSTLIB
+
+typedef struct {
+    double  X;
+    double  Y;
+    double  Z;
+} sa_XYZNumber;
+
+extern sa_XYZNumber sa_D50;
+void sa_SetUnity3x3(double mat[3][3]);
+void sa_Cpy3x3(double out[3][3], double mat[3][3]);
+void sa_MulBy3x3(double out[3], double mat[3][3], double in[3]);
+void sa_Mul3x3_2(double dst[3][3], double src1[3][3], double src2[3][3]);
+int sa_Inverse3x3(double out[3][3], double in[3][3]);
+
+#define icmXYZNumber sa_XYZNumber
+#define icmD50 sa_D50
+#define icmSetUnity3x3 sa_SetUnity3x3
+#define icmCpy3x3 sa_Cpy3x3
+#define icmMulBy3x3 sa_MulBy3x3
+#define icmMul3x3_2 sa_Mul3x3_2
+#define icmInverse3x3 sa_Inverse3x3
+
+#endif /* SALONEINSTLIB */
+/* - - - - - - - - - - - - - - - - - - -- */
+
+
 #ifdef __cplusplus
 	}
 #endif
