@@ -16,6 +16,8 @@
 
 /* TTBD
 
+    Add bell at end of readings ?
+
 	Ideally this should be changed to always create non-normalized (absolute)
 	readings, since normalised readings are not natural, but everything
 	that deals with .ti3 data then needs fixing to deal with non-normalized
@@ -83,7 +85,7 @@ void usage(char *diag, ...) {
 	icoms *icom;
 
 	fprintf(stderr,"Read a Display, Version %s\n",ARGYLL_VERSION_STR);
-	fprintf(stderr,"Author: Graeme W. Gill, licensed under the GPL Version 3\n");
+	fprintf(stderr,"Author: Graeme W. Gill, licensed under the AGPL Version 3\n");
 	if (setup_spyd2() == 2)
 		fprintf(stderr,"WARNING: This file contains a proprietary firmware image, and may not be freely distributed !\n");
 	if (diag != NULL) {
@@ -137,7 +139,8 @@ void usage(char *diag, ...) {
 	}
 	fprintf(stderr," -p                   Use projector mode (if available)\n");
 	fprintf(stderr," -y c|l               Display type, c = CRT, l = LCD\n");
-	fprintf(stderr," -k file.cal          Apply display calibration file while reading\n");
+	fprintf(stderr," -k file.cal          Load calibration file into display while reading\n");
+	fprintf(stderr," -K file.cal          Apply calibration file to test values while reading\n");
 	fprintf(stderr," -s                   Save spectral information (default don't save)\n");
 	fprintf(stderr," -P ho,vo,ss          Position test window and scale it\n");
 	fprintf(stderr,"                      ho,vi: 0.0 = left/top, 0.5 = center, 1.0 = right/bottom etc.\n");
@@ -146,7 +149,7 @@ void usage(char *diag, ...) {
 #if defined(UNIX) && !defined(__APPLE__)
 	fprintf(stderr," -n                   Don't set override redirect on test window\n");
 #endif
-	fprintf(stderr," -K                   Run instrument calibration first (used rarely)\n");
+	fprintf(stderr," -J                   Run instrument calibration first (used rarely)\n");
 	fprintf(stderr," -N                   Disable auto calibration of instrument\n");
 	fprintf(stderr," -H                   Use high resolution spectrum mode (if available)\n");
 	fprintf(stderr," -V                   Use adaptive measurement mode (if available)\n");
@@ -184,7 +187,7 @@ int main(int argc, char *argv[])
 	int wdrift = 0;						/* Flag, nz for white drift compensation */
 	int dtype = 0;						/* Display kind, 0 = default, 1 = CRT, 2 = LCD */
 	int proj = 0;						/* NZ if projector */
-	int nocal = 0;						/* Disable auto calibration */
+	int noautocal = 0;					/* Disable auto calibration */
 	int nonorm = 0;						/* Disable normalisation */
 	char ccmxname[MAXNAMEL+1] = "\000";  /* Colorimeter Correction Matrix name */
 	ccmx *cmx = NULL;					/* Colorimeter Correction Matrix */
@@ -194,6 +197,7 @@ int main(int argc, char *argv[])
 	char inname[MAXNAMEL+1] = "\000";	/* Input cgats file base name */
 	char outname[MAXNAMEL+1] = "\000";	/* Output cgats file base name */
 	char calname[MAXNAMEL+1] = "\000";	/* Calibration file name */
+	int softcal = 0;					/* nz if cal applied to values rather than hardware */
 	double cal[3][MAX_CAL_ENT];			/* Display calibration */
 	int ncal = 256;						/* number of cal entries used */
 	cgats *icg;							/* input cgats structure */
@@ -326,10 +330,13 @@ int main(int argc, char *argv[])
 					usage("-y parameter '%c' not recognised",na[0]);
 
 			/* Calibration file */
-			} else if (argv[fa][1] == 'k') {
+			} else if (argv[fa][1] == 'k' || argv[fa][1] == 'K') {
 				fa = nfa;
 				if (na == NULL) usage("Parameter expected after -k");
 				strncpy(calname,na,MAXNAMEL); calname[MAXNAMEL] = '\000';
+				softcal = 0;
+				if (argv[fa][1] == 'K')
+					softcal = 1;
 			}
 
 			/* Save spectral data */
@@ -354,12 +361,12 @@ int main(int argc, char *argv[])
 				blackbg = 1;
 
 			/* Force calibration */
-			} else if (argv[fa][1] == 'K') {
+			} else if (argv[fa][1] == 'J') {
 				docalib = 1;
 
 			/* No auto-cal */
 			} else if (argv[fa][1] == 'N') {
-				nocal = 1;
+				noautocal = 1;
 
 			/* High res mode */
 			} else if (argv[fa][1] == 'H') {
@@ -471,7 +478,7 @@ int main(int argc, char *argv[])
 	}
 
 	if (docalib) {
-		if ((rv = disprd_calibration(itype, comport, fc, dtype, proj, adaptive, nocal, disp,
+		if ((rv = disprd_calibration(itype, comport, fc, dtype, proj, adaptive, noautocal, disp,
 		                             blackbg, override, patsize, ho, vo, verb, debug)) != 0) {
 			error("docalibration failed with return value %d\n",rv);
 		}
@@ -647,10 +654,11 @@ int main(int argc, char *argv[])
 		cal[0][0] = -1.0;	/* Not used */
 	}
 
-	if ((dr = new_disprd(&errc, itype, fake ? -99 : comport, fc, dtype, proj, adaptive, nocal,
-	                     highres, 0, cal, ncal, disp, blackbg, override, ccallout, mcallout,
-	                     patsize, ho, vo, cmx != NULL ? cmx->matrix : NULL, spec, icxOT_none,
-	                     bdrift, wdrift, verb, VERBOUT, debug, "fake" ICC_FILE_EXT)) == NULL)
+	if ((dr = new_disprd(&errc, itype, fake ? -99 : comport, fc, dtype, proj, adaptive, noautocal,
+	                     highres, 0, cal, ncal, softcal, disp, blackbg, override, ccallout,
+	                     mcallout, patsize, ho, vo, cmx != NULL ? cmx->matrix : NULL,
+		                 spec, icxOT_none, bdrift, wdrift, verb, VERBOUT, debug,
+		                 "fake" ICC_FILE_EXT)) == NULL)
 		error("new_disprd failed with '%s'\n",disprd_err(errc));
 
 	if (cmx != NULL)
