@@ -10,8 +10,8 @@
  * Copyright 1996 - 2007, Graeme W. Gill
  * All rights reserved.
  *
- * This material is licenced under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 :-
- * see the License.txt file for licencing details.
+ * This material is licenced under the GNU GENERAL PUBLIC LICENSE Version 2 or later :-
+ * see the License2.txt file for licencing details.
  */
 
 /* 
@@ -38,9 +38,11 @@
 #include <string.h>
 #include <time.h>
 #include <stdarg.h>
+#ifndef SALONEINSTLIB
 #include "copyright.h"
-#include "config.h"
+#include "aconfig.h"
 #include "numlib.h"
+#endif  /* !SALONEINSTLIB */
 #include "xspect.h"
 #include "insttypes.h"
 #include "icoms.h"
@@ -947,6 +949,9 @@ dtp22_set_mode(inst *pp, inst_mode m)
 	return inst_ok;
 }
 
+/* !! It's not clear if there is a way of knowing */
+/* whether the instrument has a UV filter. */
+
 /* 
  * set or reset an optional mode
  * We assume that the instrument has been initialised.
