@@ -14,7 +14,8 @@
 
 /*
  * TTBD: 
- *
+	
+	Do we need to deterct & mark display values normalized to Y = 100 ??
  */
 
 
@@ -27,7 +28,7 @@
 #include <time.h>
 #include <string.h>
 #include "copyright.h"
-#include "config.h"
+#include "aconfig.h"
 #include "numlib.h"
 #include "cgats.h"
 #include "xicc.h"
@@ -36,7 +37,7 @@
 void usage(char *diag, ...) {
 	fprintf(stderr,"Fake test chart reader - lookup values in ICC/MPP profile, Version %s\n",
 	               ARGYLL_VERSION_STR);
-	fprintf(stderr,"Author: Graeme W. Gill, licensed under the GPL Version 3\n");
+	fprintf(stderr,"Author: Graeme W. Gill, licensed under the AGPL Version 3\n");
 	if (diag != NULL) {
 		va_list args;
 		fprintf(stderr,"  Diagnostic: ");
@@ -58,6 +59,7 @@ void usage(char *diag, ...) {
 	fprintf(stderr," -u                Make random deviations have uniform distributions rather than normal\n");
 	fprintf(stderr," -S seed           Set random seed\n");
 	fprintf(stderr," -b L,a,b          Scale black point to target Lab value\n");
+	fprintf(stderr," -I intent         r = relative colorimetric, a = absolute (default)\n");
 	fprintf(stderr," [separation.icm]  Device link separation profile\n");
 	fprintf(stderr," profile.[icc|mpp|ti3] ICC, MPP profile or TI3 to use\n");
 	fprintf(stderr," outfile           Base name for input[ti1]/output[ti3] file\n");
@@ -79,11 +81,11 @@ int main(int argc, char *argv[])
 	unsigned int seed = time(NULL);	/* Random seed value */
 	double tbp[3] = { -1.0, 0.0, 0.0 };	/* Target black point */
 	int applycal = 0;		/* NZ to apply calibration */
-	static char sepname[500] = { 0 };	/* ICC separation profile */
-	static char calname[500] = { 0 };	/* device calibration */
-	static char profname[500] = { 0 };	/* ICC or MPP Profile name */
-	static char inname[500] = { 0 };	/* Input cgats file base name */
-	static char outname[500] = { 0 };	/* Output cgats file base name */
+	static char sepname[MAXNAMEL+1] = { 0 };	/* ICC separation profile */
+	static char calname[MAXNAMEL+1] = { 0 };	/* device calibration */
+	static char profname[MAXNAMEL+1] = { 0 };	/* ICC or MPP Profile name */
+	static char inname[MAXNAMEL+1] = { 0 };	/* Input cgats file base name */
+	static char outname[MAXNAMEL+1] = { 0 };	/* Output cgats file base name */
 	cgats *icg;			/* input cgats structure */
 	cgats *ocg;			/* output cgats structure */
 	int nmask = 0;		/* Test chart device colorant mask */
@@ -107,6 +109,7 @@ int main(int argc, char *argv[])
 	xcal *cal = NULL;			/* calibration */ 
 
 	/* ICC profile based */
+	icRenderingIntent intent = icAbsoluteColorimetric;
 	icmFile *icc_fp = NULL;	/* Color profile file */
 	icc *icc_icco = NULL;	/* Profile object */
 	icmLuBase *icc_luo = NULL;	/* Conversion object */
@@ -239,6 +242,22 @@ int main(int argc, char *argv[])
 				if (tbp[2] < -128.0 || tbp[2] > 128.0) usage("-b b* value out of range");
 			}
 
+			/* Intent (only applies to ICC profile) */
+			else if (argv[fa][1] == 'I') {
+				fa = nfa;
+				if (na == NULL) usage("Expect argument to -I");
+    			switch (na[0]) {
+					case 'r':
+						intent = icRelativeColorimetric;
+						break;
+					case 'a':
+						intent = icAbsoluteColorimetric;
+						break;
+					default:
+						usage("Unexpected argument value '%c' to -I optyion",na[0]);
+				}
+			}
+
 			else
 				usage("Unrecognised flag");
 		}
@@ -249,16 +268,16 @@ int main(int argc, char *argv[])
 	/* Get the file name argument */
 	if (dosep) {
 		if (fa >= argc || argv[fa][0] == '-') usage("Missing separation profile filename argument");
-		strcpy(sepname,argv[fa++]);
+		strncpy(sepname,argv[fa++],MAXNAMEL); sepname[MAXNAMEL] = '\000';
 	}
 
 	if (fa >= argc || argv[fa][0] == '-') usage("Missing profile filename argument");
-	strcpy(profname,argv[fa++]);
+	strncpy(profname,argv[fa++],MAXNAMEL); profname[MAXNAMEL] = '\000';
 
 	if (fa >= argc || argv[fa][0] == '-') usage("Missing basename argument");
-	strcpy(inname,argv[fa]);
+	strncpy(inname,argv[fa],MAXNAMEL-4); inname[MAXNAMEL-4] = '\000';
 	strcat(inname,".ti1");
-	strcpy(outname,argv[fa]);
+	strncpy(outname,argv[fa],MAXNAMEL-4); outname[MAXNAMEL-4] = '\000';
 	strcat(outname,".ti3");
 
 	rand32(seed);		/* Init seed */
@@ -317,7 +336,7 @@ int main(int argc, char *argv[])
 		}
 
 		/* Get a Device to PCS conversion object */
-		if ((icc_luo = icc_icco->get_luobj(icc_icco, icmFwd, icAbsoluteColorimetric,
+		if ((icc_luo = icc_icco->get_luobj(icc_icco, icmFwd, intent,
 		                           dolab ? icSigLabData : icSigXYZData, icmLuOrdNorm)) == NULL) {
 			if ((icc_luo = icc_icco->get_luobj(icc_icco, icmFwd, icmDefaultIntent,
 			                       dolab ? icSigLabData : icSigXYZData, icmLuOrdNorm)) == NULL)
@@ -379,7 +398,7 @@ int main(int argc, char *argv[])
 			}
 
 			mlu->get_info(mlu, &cnv_nmask, &inn, NULL,
-			                   &spec_n, &spec_wl_short, &spec_wl_long, &itype);
+			                   &spec_n, &spec_wl_short, &spec_wl_long, &itype, NULL);
 	
 			outn = 3;
 			outs = dolab ? icSigLabData : icSigXYZData;
@@ -397,7 +416,7 @@ int main(int argc, char *argv[])
 
 	/* If we don't have an ICC or MPP lookup object, look for a TI3 */
 	if (icc_luo == NULL && mlu == NULL) {
-		char *buf, *outc;
+		char *rbuf, *outc;
 		char *ti3_bident;
 		int ti3_nchan;
 
@@ -413,13 +432,13 @@ int main(int argc, char *argv[])
 			error ("Input file '%s' doesn't contain one table",profname);
 
 		if ((ti = ti3->find_kword(ti3, 0, "COLOR_REP")) < 0)
-			error("Input file doesn't contain keyword COLOR_REPS");
+			error("Input file doesn't contain keyword COLOR_REP");
 	
-		if ((buf = strdup(ti3->t[0].kdata[ti])) == NULL)
+		if ((rbuf = strdup(ti3->t[0].kdata[ti])) == NULL)
 			error("Malloc failed");
 	
 		/* Split COLOR_REP into device and PCS space */
-		if ((outc = strchr(buf, '_')) == NULL)
+		if ((outc = strchr(rbuf, '_')) == NULL)
 			error("COLOR_REP '%s' invalid", ti3->t[0].kdata[ti]);
 		*outc++ = '\000';
 	
@@ -433,11 +452,11 @@ int main(int argc, char *argv[])
 		} else
 			error("COLOR_REP '%s' invalid (Neither XYZ nor LAB)", ti3->t[0].kdata[ti]);
 
-		if ((cnv_nmask = icx_char2inkmask(buf)) == 0) {
-			error ("File '%s' keyword COLOR_REPS has unknown device value '%s'",profname,buf);
+		if ((cnv_nmask = icx_char2inkmask(rbuf)) == 0) {
+			error ("File '%s' keyword COLOR_REP has unknown device value '%s'",profname,rbuf);
 		}
 
-		free(buf);
+		free(rbuf);
 
 		if ((ins = icx_colorant_comb_to_icc(cnv_nmask)) == 0)
 			error ("Couldn't match MPP mask to valid ICC colorspace");
@@ -480,6 +499,8 @@ int main(int argc, char *argv[])
 
 		/* Find spectral fields */
 		if ((ti = ti3->find_kword(ti3, 0, "SPECTRAL_BANDS")) >= 0) {
+			char buf[100];
+
 			spec_n = atoi(ti3->t[0].kdata[ti]);
 			if ((ti = ti3->find_kword(ti3, 0, "SPECTRAL_START_NM")) < 0)
 				error ("Input file doesn't contain keyword SPECTRAL_START_NM");
@@ -520,7 +541,7 @@ int main(int argc, char *argv[])
 			printf("Adding %.3f%% average deviation %s noise to device values\n",rdlevel * 100.0,unidist ? "uniform" : "normal");
 		for (j = 0; j < inn && j < 10; j++) {
 			if (chpow[j] != 1.0)
-				printf("Appluing chan %d power %f\n",j,chpow[j]);
+				printf("Applying chan %d power %f\n",j,chpow[j]);
 		}
 		if (rplevel > 0.0)
 			printf("Adding %.3f%% average deviation %s noise to %s PCS values\n",rplevel * 100.0,unidist ? "uniform" : "normal", dolab ? "L*a*b*": "XYZ");
@@ -542,6 +563,13 @@ int main(int argc, char *argv[])
 	if ((npat = icg->t[0].nsets) <= 0)
 		error ("No sets of data");
 
+	/* Figure out the color space of the .ti1 */
+	if ((fi = icg->find_kword(icg, 0, "COLOR_REP")) < 0)
+		error ("Input file doesn't contain keyword COLOR_REP");
+
+	if ((nmask = icx_char2inkmask(icg->t[0].kdata[fi])) == 0)
+		error ("Input file keyword COLOR_REP has unknown value");
+
 	/* Setup output cgats file */
 	ocg = new_cgats();				/* Create a CGATS structure */
 	ocg->add_other(ocg, "CTI3"); 		/* our special type is Calibration Target Information 3 */
@@ -559,15 +587,25 @@ int main(int argc, char *argv[])
 		ocg->add_kword(ocg, 0, "DEVICE_CLASS",ti3->t[0].kdata[ti], NULL);	/* Copy */
 	} else if (mlu != NULL) {
 		/* This is a guess. It may not be correct. */
-		if (cnv_nmask & ICX_ADDITIVE)	
+		if ((cnv_nmask & ICX_ADDITIVE) ^ (cnv_nmask & ICX_INVERTED)) {	
 			ocg->add_kword(ocg, 0, "DEVICE_CLASS","DISPLAY", NULL);
-		else
+			if (nmask == ICX_IRGB)
+				warning("It's unusual to have an iRGB display device !");
+		} else {
 			ocg->add_kword(ocg, 0, "DEVICE_CLASS","OUTPUT", NULL);
+			if (nmask == ICX_RGB)
+				warning("It's unusual to have an RGB printing device !");
+		}
 	} else {	/* Assume ICC */
-		if (icc_icco->header->deviceClass == icSigDisplayClass)
+		if (icc_icco->header->deviceClass == icSigDisplayClass) {
 			ocg->add_kword(ocg, 0, "DEVICE_CLASS","DISPLAY", NULL);
-		else
+			if (nmask == ICX_IRGB)
+				warning("It's unusual to have an iRGB display device !");
+		} else {
 			ocg->add_kword(ocg, 0, "DEVICE_CLASS","OUTPUT", NULL);
+			if (nmask == ICX_RGB)
+				warning("It's unusual to have an RGB printing device !");
+		}
 	}
 
 	if ((ti = icg->find_kword(icg, 0, "SINGLE_DIM_STEPS")) >= 0)
@@ -592,13 +630,6 @@ int main(int argc, char *argv[])
 	if ((si = icg->find_field(icg, 0, "SAMPLE_ID")) < 0)
 		error ("Input file doesn't contain field SAMPLE_ID");
 
-	/* Figure out the color space */
-	if ((fi = icg->find_kword(icg, 0, "COLOR_REP")) < 0)
-		error ("Input file doesn't contain keyword COLOR_REPS");
-
-	if ((nmask = icx_char2inkmask(icg->t[0].kdata[fi])) == 0)
-		error ("Input file keyword COLOR_REPS has unknown value");
-
 	{
 		int i, j, ii;
 		int chix[ICX_MXINKS];	/* Device chanel indexes */
@@ -657,7 +688,7 @@ int main(int argc, char *argv[])
 			}
 		} else if (mlu != NULL) {
 			/* Check if mpp is compatible with .ti1 */
-			if (nmask == ICX_W && cnv_nmask == ICX_RGB)
+			if (nmask == ICX_W && (cnv_nmask == ICX_RGB || cnv_nmask == ICX_IRGB))
 				gfudge = 1;
 			else if (nmask == ICX_K && (cnv_nmask & ICX_BLACK))
 				gfudge = 2;
@@ -666,7 +697,7 @@ int main(int argc, char *argv[])
 				       icx_inkmask2char(cnv_nmask, 1), ident);	// Should free().
 		} else if (ti3 != NULL) {
 			/* Check if .ti3 is compatible with .ti1 */
-			if (nmask == ICX_W && cnv_nmask == ICX_RGB)
+			if (nmask == ICX_W && (cnv_nmask == ICX_RGB || cnv_nmask == ICX_IRGB))
 				gfudge = 1;
 			else if (nmask == ICX_K && (cnv_nmask & ICX_BLACK))
 				gfudge = 2;
