@@ -10,8 +10,8 @@
  * Copyright 2007, Graeme W. Gill
  * All rights reserved.
  *
- * This material is licenced under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 :-
- * see the License.txt file for licencing details.
+ * This material is licenced under the GNU GENERAL PUBLIC LICENSE Version 2 or later :-
+ * see the License2.txt file for licencing details.
  */
 
 /* 
@@ -38,16 +38,21 @@
 #include <string.h>
 #include <time.h>
 #include <stdarg.h>
+#ifndef SALONEINSTLIB
 #include "copyright.h"
-#include "config.h"
+#include "aconfig.h"
 #include "numlib.h"
+#else /* SALONEINSTLIB */
+#include "sa_config.h"
+#include "numsup.h"
+#endif /* SALONEINSTLIB */
 #include "xspect.h"
 #include "insttypes.h"
 #include "icoms.h"
 #include "conv.h"
 #include "hcfr.h"
 
-#undef MEASURE_RAW		/* To facilitie calibration - return raw RGB values */
+#undef MEASURE_RAW		/* To facilitate calibration - return raw RGB values */
 #undef DEBUG
 
 static inst_code hcfr_interp_code(inst *pp, int ec);
@@ -139,7 +144,7 @@ hcfr_flush(
 
 	for (c->lerr = 0;;) {
 		int debug = c->debug; c->debug = 0;
-		c->read(c, buf, MAX_MES_SIZE, '\000', 100000, 0.01);
+		c->read(c, buf, MAX_MES_SIZE, '\000', 100000, 0.05);
 		c->debug = debug;
 		if (c->lerr != 0)
 			break;				/* Expect timeout with nothing to read */
@@ -404,7 +409,7 @@ hcfr_init_coms(inst *pp, int port, baud_rate br, flow_control fc, double tout) {
 
 	/* Set config, interface, "Serial" write & read end points */
 	/* Note if we clear halt the interface hangs */
-	p->icom->set_usb_port(p->icom, port, 1, 0x03, 0x83, usbflags, 0); 
+	p->icom->set_usb_port(p->icom, port, 1, 0x03, 0x83, usbflags, 0, NULL); 
 
 	if ((ev = hcfr_break(p)) != inst_ok) {
 		if (p->debug) fprintf(stderr,"hcfr: Error doing break\n");
@@ -580,6 +585,10 @@ ipatch *val) {		/* Pointer to instrument patch value */
 		val->aXYZ[1] = rgb[1];
 		val->aXYZ[2] = rgb[2];
 	}
+
+	/* Apply the colorimeter correction matrix */
+	icmMulBy3x3(val->aXYZ, p->ccmat, val->aXYZ);
+
 	val->aXYZ_v = 1;		/* These are absolute XYZ readings */
 	val->XYZ_v = 0;
 	val->Lab_v = 0;
@@ -591,6 +600,23 @@ ipatch *val) {		/* Pointer to instrument patch value */
 	return inst_ok;
 }
 
+/* Insert a colorimetric correction matrix in the instrument XYZ readings */
+/* This is only valid for colorimetric instruments. */
+/* To remove the matrix, pass NULL for the filter filename */
+inst_code hcfr_col_cor_mat(
+inst *pp,
+double mtx[3][3]
+) {
+	hcfr *p = (hcfr *)pp;
+
+	if (mtx == NULL)
+		icmSetUnity3x3(p->ccmat);
+	else
+		icmCpy3x3(p->ccmat, mtx);
+		
+	return inst_ok;
+}
+
 /* Determine if a calibration is needed. Returns inst_calt_none if not, */
 /* inst_calt_unknown if it is unknown, or inst_calt_XXX if needs calibration, */
 /* and the first type of calibration needed. */
@@ -736,6 +762,7 @@ inst_capability hcfr_capabilities(inst *pp) {
 	rv = inst_emis_spot
 	   | inst_emis_disp
 	   | inst_colorimeter
+	   | inst_ccmx
 	   | inst_emis_disp_crt
 	   | inst_emis_disp_lcd
 	   ;
@@ -829,6 +856,8 @@ extern hcfr *new_hcfr(icoms *icom, int debug, int verb)
 	p->debug = debug;
 	p->verb = verb;
 
+	icmSetUnity3x3(p->ccmat);	/* Set the colorimeter correction matrix to do nothing */
+
 	p->init_coms        = hcfr_init_coms;
 	p->init_inst        = hcfr_init_inst;
 	p->capabilities     = hcfr_capabilities;
@@ -847,6 +876,7 @@ extern hcfr *new_hcfr(icoms *icom, int debug, int verb)
 	p->needs_calibration = hcfr_needs_calibration;
 	p->calibrate        = hcfr_calibrate;
 	p->comp_filter    	= hcfr_comp_filter;
+	p->col_cor_mat      = hcfr_col_cor_mat;
 	p->interp_error     = hcfr_interp_error;
 	p->inst_interp_error = NULL;				/* virtual constructor will do this */
 	p->last_comerr      = hcfr_last_comerr;
