@@ -10,8 +10,8 @@
  * Copyright 2001 - 2010 Graeme W. Gill
  * All rights reserved.
  *
- * This material is licenced under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 :-
- * see the License.txt file for licencing details.
+ * This material is licenced under the GNU GENERAL PUBLIC LICENSE Version 2 or later :-
+ * see the License2.txt file for licencing details.
  */
 
 #include <stdio.h>
@@ -19,8 +19,10 @@
 #include <ctype.h>
 #include <string.h>
 #include <time.h>
+#ifndef SALONEINSTLIB
 #include "copyright.h"
 #include "aconfig.h"
+#endif /* !SALONEINSTLIB */
 #include "xspect.h"
 #include "insttypes.h"
 
@@ -135,8 +137,8 @@ unsigned short idProduct) {
 			return instDTP92;
 	  	if (idProduct == 0xD094)	/* DTP94 */
 			return instDTP94;
-//	  	if (idProduct == 0x5001)	/* HueyPro */
-//			return instHuey;
+	  	if (idProduct == 0x5001)	/* HueyL (Lenovo W70DS Laptop ?) */
+			return instHuey;
 	}
 
 	if (idVendor  == 0x0971) {		/* Gretag Macbeth */
@@ -164,6 +166,8 @@ unsigned short idProduct) {
 	}
 
 	if (idVendor  == 0x085C) {
+		if (idProduct == 0x0100)	/* ColorVison Spyder1 */
+			return instSpyder2;		/* Alias to Spyder 2 */
 		if (idProduct == 0x0200)	/* ColorVison Spyder2 */
 			return instSpyder2;
 		if (idProduct == 0x0300)	/* ColorVison Spyder3 */
@@ -216,9 +220,10 @@ int inst_illuminant(xspect *sp, instType itype) {
 		case instSpectroScanT:
 			return standardIlluminant(sp, icxIT_A, 0);		/* Standard A type assumed */
 
+#ifndef SALONEINSTLIB
 		case instSpectrocam:
 			return standardIlluminant(sp, icxIT_Spectrocam, 0);   /* Spectrocam Xenon Lamp */
-
+#endif
 		case instI1Display:
 			return 1;										/* Not applicable */
 
