@@ -1,17 +1,17 @@
 
+ /* X-Rite ColorMunki related functions */
+
 /* 
  * Argyll Color Correction System
  *
- * X-Rite ColorMunki related functions
- *
  * Author: Graeme W. Gill
  * Date:   12/1/2009
  *
- * Copyright 2006 - 2009, Graeme W. Gill
+ * Copyright 2006 - 2010, Graeme W. Gill
  * All rights reserved.
  *
- * This material is licenced under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 :-
- * see the License.txt file for licencing details.
+ * This material is licenced under the GNU GENERAL PUBLIC LICENSE Version 2 or later :-
+ * see the License2.txt file for licencing details.
  * 
  * (Based on i1pro.c)
  */
@@ -41,9 +41,11 @@
 #include <time.h>
 #include <stdarg.h>
 #include <math.h>
+#ifndef SALONEINSTLIB
 #include "copyright.h"
-#include "config.h"
+#include "aconfig.h"
 #include "numlib.h"
+#endif	/* !SALONEINSTLIB */
 #include "xspect.h"
 #include "insttypes.h"
 #include "icoms.h"
@@ -77,9 +79,22 @@ munki_init_coms(inst *pp, int port, baud_rate br, flow_control fc, double tout)
 	int rsize;
 	long etime;
 	int bi, i, rv;
-	int retries = 0;
 	inst_code ev = inst_ok;
 	icomuflags usbflags = icomuf_none;
+#ifdef __APPLE__
+	/* If the ColorMunki software has been installed, then there will */
+	/* be a daemon process that has the device open. Kill that process off */
+	/* so that we can open it here, before it re-spawns. */
+	char *pnames[] = {
+			"ninjad",
+			"ColorMunkiDeviceService",
+			NULL
+	};
+	int retries = 20;
+#else /* !__APPLE__ */
+	char **pnames = NULL;
+	int retries = 0;
+#endif /* !__APPLE__ */
 
 	if (p->debug) {
 		p->icom->debug = p->debug;	/* Turn on debugging */
@@ -94,31 +109,9 @@ munki_init_coms(inst *pp, int port, baud_rate br, flow_control fc, double tout)
 
 	if (p->debug) fprintf(stderr,"munki: About to init USB\n");
 
-#ifdef __APPLE__
-	/* If the ColorMunki software has been installed, then there will */
-	/* be a daemon process that has the device open. Kill that process off */
-	/* so that we can open it here, before it re-spawns. */
-
-	if ((rv = kill_nprocess("ninjad")) < 0) {
-		if (p->debug) fprintf(stderr,"munki: killing ninjad process failed!\n");
-	} else if (rv > 0) {
-		retries = 10;			/* Wait for process to die */
-	}
-#endif /* __APPLE__ */
-
-/* Linix + munki bug workaround */
-/* Note that the munki rev D seems to crash on any get_configuration, */
-/* and is slow to release_interface under Linux (several seconds). */
-/* It also dissapears if closed under Linux, so a reset is used instead. */
-/* Not releasing the interface for the rev. D seems to cause Linix to crash ! */
-/* This may be specific to some version of Linix USB stack only ? */
-#if defined(UNIX) && !defined(__APPLE__)
-	usbflags |= icomuf_reset_not_close;
-#endif
-
 	/* Set config, interface, write end point, read end point, read quanta */
 	/* ("serial" end points aren't used - the Munki uses USB control messages) */
-	p->icom->set_usb_port(p->icom, port, 1, 0x00, 0x00, usbflags, retries); 
+	p->icom->set_usb_port(p->icom, port, 1, 0x00, 0x00, usbflags, retries, pnames); 
 
 	if (p->debug) fprintf(stderr,"munki: init coms has suceeded\n");
 
@@ -148,7 +141,7 @@ munki_init_inst(inst *pp) {
 	       |  inst_emis_spot
 	       |  inst_emis_disp
 	       |  inst_emis_proj		/* Support of a specific projector mode */
-	       |  inst_emis_illum
+	       |  inst_emis_tele
 	       |  inst_trans_spot		/* Support this manually using a light table */
 	       |  inst_trans_strip 
 	       |  inst_emis_strip		/* Also likely to be light table reading */
@@ -178,6 +171,11 @@ munki_init_inst(inst *pp) {
 	return munki_interp_code(p, ev);
 }
 
+static char *munki_get_serial_no(inst *pp) {
+	munki *p = (munki *)pp;
+	return munki_imp_get_serial_no(p);
+}
+
 /* Read a set of strips */
 /* Return the dtp error code */
 static inst_code
@@ -378,7 +376,6 @@ munki_interp_error(inst *pp, munki_code ec) {
 			return "Unable to save calibration to file";
 		case MUNKI_INT_CAL_RESTORE:
 			return "Unable to restore calibration from file";
-
 		default:
 			return "Unknown error code";
 	}
@@ -527,13 +524,18 @@ inst_code munki_set_mode(inst *pp, inst_mode m) {
 			return inst_unsupported;
 		}
 	} else if ((mm & inst_mode_illum_mask) == inst_mode_emission) {
-		if ((mm & inst_mode_sub_mask) == inst_mode_disp) {
-			mmode = mk_disp_spot;
-		} else if ((mm & inst_mode_sub_mask) == inst_mode_proj) {
-			mmode = mk_proj_spot;
-		} else if ((mm & inst_mode_sub_mask) == inst_mode_spot
-		        || (mm & inst_mode_sub_mask) == inst_mode_illum) {
-			mmode = mk_emiss_spot;
+		if ((mm & inst_mode_sub_mask) == inst_mode_spot) {
+			if ((mm & inst_mode_mod_mask) == inst_mode_disp) {
+				mmode = mk_disp_spot;
+			} else {
+				mmode = mk_emiss_spot;
+			}
+		} else if ((mm & inst_mode_sub_mask) == inst_mode_tele) {
+			if ((mm & inst_mode_mod_mask) == inst_mode_disp) {
+				mmode = mk_proj_spot;
+			} else {
+				mmode = mk_tele_spot;
+			}
 		} else if ((mm & inst_mode_sub_mask) == inst_mode_strip) {
 			mmode = mk_emiss_scan;
 		} else if ((mm & inst_mode_sub_mask) == inst_mode_ambient) {
@@ -587,6 +589,22 @@ inst_status_type m,	/* Requested status type */
 		return inst_ok;
 	}
 
+	/* Return the filter */
+	else if (m == inst_stat_get_filter) {
+		munki_code ev;
+		inst_opt_filter *filt;
+		va_list args;
+
+		va_start(args, m);
+		filt = va_arg(args, inst_opt_filter *);
+		va_end(args);
+
+		/* The ColorMunki is always UV cut */
+		*filt = inst_opt_filter_UVCut;
+
+		return inst_ok;
+	}
+
 	return inst_unsupported;
 }
 
@@ -738,6 +756,7 @@ munki_set_opt_mode(inst *pp, inst_opt_mode m, ...)
 		if (trans_time_prop != NULL) *trans_time_prop = p->led_trans_time_prop;
 		return inst_ok;
 	}
+
 	return inst_unsupported;
 }
 
@@ -775,6 +794,7 @@ extern munki *new_munki(icoms *icom, int debug, int verb)
 	/* Inst methods */
 	p->init_coms         = munki_init_coms;
 	p->init_inst         = munki_init_inst;
+	p->get_serial_no     = munki_get_serial_no;
 	p->capabilities      = munki_capabilities;
 	p->capabilities2     = munki_capabilities2;
 	p->set_mode          = munki_set_mode;
