@@ -12,8 +12,8 @@
  * Copyright 2001 - 2010 Graeme W. Gill
  * All rights reserved.
  *
- * This material is licenced under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 :-
- * see the License.txt file for licencing details.
+ * This material is licenced under the GNU GENERAL PUBLIC LICENSE Version 2 or later :-
+ * see the License2.txt file for licencing details.
  */
 
 /* This program reads a spot reflection/transmission/emission value using */
@@ -25,6 +25,8 @@
  *  so that it can be sent a graph without needing to be clicked in all the time.
  *
  *  Should add option to show reflective/tranmsission density values.
+ *
+ *  Should add option for Y u' v' values.
  */
 
 #undef DEBUG
@@ -35,8 +37,10 @@
 #include <stdlib.h>
 #include <math.h>
 #include <sys/types.h>
+#include <fcntl.h>
 #include <time.h>
 #include <string.h>
+#ifndef SALONEINSTLIB
 #include "copyright.h"
 #include "aconfig.h"
 #include "numlib.h"
@@ -44,20 +48,24 @@
 #include "xicc.h"
 #include "ccmx.h"
 #include "conv.h"
+#include "plot.h"
+#else /* SALONEINSTLIB */
+#include "sa_config.h"
+#include "numsup.h"
+#include "xspect.h"
+#include "conv.h"
+#endif /* SALONEINSTLIB */
 #include "inst.h"
 #include "icoms.h"
-#include "plot.h"
-#include "spyd2setup.h"			/* Enable Spyder 2 access */
-
-#include <stdarg.h>
+#include "spyd2setup.h"
+#ifdef EN_SPYD2
+#include "spyd2setup.h"
+#endif
 
 #if defined (NT)
 #include <conio.h>
 #endif
 
-# define RELEASE_VERSION ARGYLL_VERSION
-# define RELEASE_VERSION_STR ARGYLL_VERSION_STR
-
 #ifdef NEVER	/* Not currently used */
 /* Convert control chars to ^[A-Z] notation in a string */
 static char *
@@ -77,6 +85,90 @@ fix_asciiz(char *s) {
 }
 #endif
 
+#ifdef SALONEINSTLIB
+
+#define D50_X_100  96.42
+#define D50_Y_100 100.00
+#define D50_Z_100  82.49
+
+/* CIE XYZ 0..100 to perceptual D50 CIE 1976 L*a*b* */
+static void
+XYZ2Lab(double *out, double *in) {
+	double X = in[0], Y = in[1], Z = in[2];
+	double x,y,z,fx,fy,fz;
+
+	x = X/D50_X_100;
+	y = Y/D50_Y_100;
+	z = Z/D50_Z_100;
+
+	if (x > 0.008856451586)
+		fx = pow(x,1.0/3.0);
+	else
+		fx = 7.787036979 * x + 16.0/116.0;
+
+	if (y > 0.008856451586)
+		fy = pow(y,1.0/3.0);
+	else
+		fy = 7.787036979 * y + 16.0/116.0;
+
+	if (z > 0.008856451586)
+		fz = pow(z,1.0/3.0);
+	else
+		fz = 7.787036979 * z + 16.0/116.0;
+
+	out[0] = 116.0 * fy - 16.0;
+	out[1] = 500.0 * (fx - fy);
+	out[2] = 200.0 * (fy - fz);
+}
+
+/* Return the normal Delta E given two Lab values */
+static double LabDE(double *Lab0, double *Lab1) {
+	double rv = 0.0, tt;
+
+	tt = Lab0[0] - Lab1[0];
+	rv += tt * tt;
+	tt = Lab0[1] - Lab1[1];
+	rv += tt * tt;
+	tt = Lab0[2] - Lab1[2];
+	rv += tt * tt;
+	
+	return sqrt(rv);
+}
+
+/* Lab to LCh */
+void Lab2LCh(double *out, double *in) {
+	double C, h;
+
+	C = sqrt(in[1] * in[1] + in[2] * in[2]);
+
+    h = (180.0/3.14159265359) * atan2(in[2], in[1]);
+	h = (h < 0.0) ? h + 360.0 : h;
+
+	out[0] = in[0];
+	out[1] = C;
+	out[2] = h;
+}
+
+/* XYZ to Yxy */
+static void XYZ2Yxy(double *out, double *in) {
+	double sum = in[0] + in[1] + in[2];
+	double Y, x, y;
+
+	if (sum < 1e-9) {
+		Y = 0.0;
+		y = 0.0;
+		x = 0.0;
+	} else {
+		Y = in[1];
+		x = in[0]/sum;
+		y = in[1]/sum;
+	}
+	out[0] = Y;
+	out[1] = x;
+	out[2] = y;
+}
+
+#endif /* SALONEINSTLIB */
 
 /* Replacement for gets */
 char *getns(char *buf, int len) {
@@ -137,14 +229,16 @@ static int gcc_bug_fix(int i) {
 void
 usage(int debug) {
 	icoms *icom;
-	fprintf(stderr,"Read Print Spot values, Version %s\n",RELEASE_VERSION_STR);
-	fprintf(stderr,"Author: Graeme W. Gill, licensed under the GPL Version 3\n");
+	fprintf(stderr,"Read Print Spot values, Version %s\n",ARGYLL_VERSION_STR);
+	fprintf(stderr,"Author: Graeme W. Gill, licensed under the GPL Version 2 or later\n");
 	if (setup_spyd2() == 2)
 		fprintf(stderr,"WARNING: This file contains a proprietary firmware image, and may not be freely distributed !\n");
 	fprintf(stderr,"usage: spotread [-options] [logfile]\n");
 	fprintf(stderr," -v                   Verbose mode\n");
 	fprintf(stderr," -s                   Print spectrum for each reading\n");
+#ifndef SALONEINSTLIB
 	fprintf(stderr," -S                   Plot spectrum for each reading\n");
+#endif /* !SALONEINSTLIB */
 	fprintf(stderr," -c listno            Set communication port from the following list (default %d)\n",COMPORT);
 	if ((icom = new_icoms()) != NULL) {
 		icompath **paths;
@@ -180,10 +274,20 @@ usage(int debug) {
 	fprintf(stderr," -f                   Use ambient flash measurement mode (absolute results)\n");
 	fprintf(stderr," -y c|l               Display type (if emissive), c = CRT, l = LCD\n");
 	fprintf(stderr," -i illum             Choose illuminant for print/transparency spectral data:\n");
+#ifndef SALONEINSTLIB
 	fprintf(stderr,"                      A, C, D50 (def.), D65, F5, F8, F10 or file.sp\n");
+#else
+	fprintf(stderr,"                      A, C, D50 (def.), D65\n");
+#endif
 	fprintf(stderr," -o observ            Choose CIE Observer for spectral data:\n");
+#ifndef SALONEINSTLIB
 	fprintf(stderr,"                      1931_2 (def), 1964_10, S&B 1955_2, shaw, J&V 1978_2\n");
+#else
+	fprintf(stderr,"                      1931_2 (def), 1964_10\n");
+#endif
+#ifndef SALONEINSTLIB
 	fprintf(stderr,"                      (Choose FWA during operation)\n");
+#endif
 	fprintf(stderr," -F filter            Set filter configuration (if aplicable):\n");
 	fprintf(stderr,"    n                  None\n");
 	fprintf(stderr,"    p                  Polarising filter\n");
@@ -191,11 +295,16 @@ usage(int debug) {
 	fprintf(stderr,"    u                  U.V. Cut\n");
 	fprintf(stderr," -E extrafilterfile   Apply extra filter compensation file\n");
 	fprintf(stderr," -x                   Display Yxy instead of Lab\n");
+	fprintf(stderr," -h                   Display LCh instead of Lab\n");
+#ifndef SALONEINSTLIB
 	fprintf(stderr," -T                   Display correlated color temperatures and CRI\n");
+#endif /* !SALONEINSTLIB */
 //	fprintf(stderr," -K type              Run instrument calibration first\n");
 	fprintf(stderr," -N                   Disable auto calibration of instrument\n");
 	fprintf(stderr," -H                   Start in high resolution spectrum mode (if available)\n");
+#ifndef SALONEINSTLIB
 	fprintf(stderr," -X file.ccmx         Apply Colorimeter Correction Matrix\n");
+#endif
 	fprintf(stderr," -W n|h|x             Override serial port flow control: n = none, h = HW, x = Xon/Xoff\n");
 	fprintf(stderr," -D [level]           Print debug diagnostics to stderr\n");
 	fprintf(stderr," logfile              Optional file to save reading results as text\n");
@@ -221,13 +330,16 @@ int main(int argc, char *argv[])
 	int ambient = 0;				/* 1 = Use ambient emissive mode, 2 = ambient flash mode */
 	int highres = 0;				/* Use high res mode if available */
 	int doYxy= 0;					/* Display Yxy instead of Lab */
+	int doLCh= 0;					/* Display LCh instead of Lab */
 	int doCCT= 0;					/* Display correlated color temperatures */
 	inst_mode mode = 0, smode = 0;	/* Normal mode and saved readings mode */
 	inst_opt_mode trigmode = inst_opt_unknown;	/* Chosen trigger mode */
 	inst_opt_filter fe = inst_opt_filter_unknown;
 									/* Filter configuration */
 	char outname[MAXNAMEL+1] = "\000";  /* Output logfile name */
+#ifndef SALONEINSTLIB
 	char ccmxname[MAXNAMEL+1] = "\000";  /* Colorimeter Correction Matrix name */
+#endif
 	char filtername[MAXNAMEL+1] = "\000";  /* Filter compensation */
 	FILE *fp = NULL;				/* Logfile */
 	int comport = COMPORT;			/* COM port used */
@@ -253,6 +365,7 @@ int main(int argc, char *argv[])
 	double Lab[3] = { -10.0, 0, 0};	/* Last Lab */
 	double rLab[3] = { -10.0, 0, 0};	/* Reference Lab */
 	double Yxy[3] = { 0.0, 0, 0};	/* Yxy value */
+	double LCh[3] = { 0.0, 0, 0};	/* LCh value */
 	int savdrd = 0;					/* At least one saved reading is available */
 	int ix;							/* Reading index number */
 	int loghead = 0;				/* NZ if log file heading has been printed */
@@ -294,8 +407,10 @@ int main(int argc, char *argv[])
 			} else if (argv[fa][1] == 's') {
 				pspec = 1;
 
+#ifndef SALONEINSTLIB
 			} else if (argv[fa][1] == 'S') {
 				pspec = 2;
+#endif /* !SALONEINSTLIB */
 
 			/* COM port  */
 			} else if (argv[fa][1] == 'c' || argv[fa][1] == 'C') {
@@ -331,6 +446,7 @@ int main(int argc, char *argv[])
 				} else if (strcmp(na, "D65") == 0) {
 					spec = 1;
 					illum = icxIT_D65;
+#ifndef SALONEINSTLIB
 				} else if (strcmp(na, "F5") == 0) {
 					spec = 1;
 					illum = icxIT_F5;
@@ -346,6 +462,10 @@ int main(int argc, char *argv[])
 					if (read_xspect(&cust_illum, na) != 0)
 						usage(debug);
 				}
+#else /* SALONEINSTLIB */
+				} else
+					usage(debug);
+#endif /* SALONEINSTLIB */
 
 			/* Spectral Observer type */
 			} else if (argv[fa][1] == 'o' || argv[fa][1] == 'O') {
@@ -357,6 +477,7 @@ int main(int argc, char *argv[])
 				} else if (strcmp(na, "1964_10") == 0) {	/* Classic 10 degree */
 					spec = 1;
 					observ = icxOT_CIE_1964_10;
+#ifndef SALONEINSTLIB
 				} else if (strcmp(na, "1955_2") == 0) {		/* Stiles and Burch 1955 2 degree */
 					spec = 1;
 					observ = icxOT_Stiles_Burch_2;
@@ -366,6 +487,7 @@ int main(int argc, char *argv[])
 				} else if (strcmp(na, "shaw") == 0) {		/* Shaw and Fairchilds 1997 2 degree */
 					spec = 1;
 					observ = icxOT_Shaw_Fairchild_2;
+#endif /* !SALONEINSTLIB */
 				} else
 					usage(debug);
 
@@ -462,10 +584,18 @@ int main(int argc, char *argv[])
 			/* Show Yxy */
 			} else if (argv[fa][1] == 'x') {
 				doYxy = 1;
+				doLCh = 0;
+
+			/* Show LCh */
+			} else if (argv[fa][1] == 'h') {
+				doYxy = 0;
+				doLCh = 1;
+#ifndef SALONEINSTLIB
 
 			/* Show CCT etc. */
 			} else if (argv[fa][1] == 'T') {
 				doCCT = 1;
+#endif /* !SALONEINSTLIB */
 
 			/* Manual calibration */
 			} else if (argv[fa][1] == 'K') {
@@ -484,11 +614,13 @@ int main(int argc, char *argv[])
 			} else if (argv[fa][1] == 'H') {
 				highres = 1;
 
+#ifndef SALONEINSTLIB
 			/* Colorimeter Correction Matrix */
 			} else if (argv[fa][1] == 'X') {
 				fa = nfa;
 				if (na == NULL) usage(debug);
 				strncpy(ccmxname,na,MAXNAMEL-1); ccmxname[MAXNAMEL-1] = '\000';
+#endif
 
 			/* Serial port flow control */
 			} else if (argv[fa][1] == 'W') {
@@ -524,6 +656,7 @@ int main(int argc, char *argv[])
 	}
 
 
+#ifndef SALONEINSTLIB
 	/* See if there is an environment variable ccmx */
 	{
 		char *na = getenv("ARGYLL_COLMTER_COR_MATRIX");
@@ -531,6 +664,7 @@ int main(int argc, char *argv[])
 			strncpy(ccmxname,na,MAXNAMEL-1); ccmxname[MAXNAMEL-1] = '\000';
 		}
 	}
+#endif
 
 	/* - - - - - - - - - - - - - - - - - - -  */
 	/* Setup the instrument ready to do reads */
@@ -724,8 +858,7 @@ int main(int argc, char *argv[])
 			if ((rv = it->set_opt_mode(it,inst_opt_noautocalib)) != inst_ok) {
 				printf("Setting no-autocalibrate failed failed with '%s' (%s)\n",
 			       it->inst_interp_error(it, rv), it->interp_error(it, rv));
-				it->del(it);
-				return -1;
+				printf("Disable auto-calibrate not supported\n");
 			}
 		}
 		if (highres) {
@@ -787,6 +920,7 @@ int main(int argc, char *argv[])
 			}
 		}
 
+#ifndef SALONEINSTLIB
 		/* Colorimeter Correction Matrix */
 		if (ccmxname[0] != '\000') {
 			ccmx *cmx;
@@ -817,6 +951,7 @@ int main(int argc, char *argv[])
 			}
 			cmx->del(cmx);
 		}
+#endif
 
 		/* If it batter powered, show the status of the battery */
 		if ((cap2 & inst2_has_battery)) {
@@ -895,7 +1030,9 @@ int main(int argc, char *argv[])
 
 		/* Create a spectral conversion object */
 		if ((sp2cie = new_xsp2cie(illum, &cust_illum, observ, NULL
+#ifndef SALONEINSTLIB
 		     , icSigXYZData
+#endif /* !SALONEINSTLIB */
 		                      )) == NULL)
 			error("Creation of spectral conversion object failed");
 	}
@@ -919,8 +1056,10 @@ int main(int argc, char *argv[])
 		ipatch val;
 		double XYZ[3];			/* XYZ scaled 0..100 or absolute */
 		double tXYZ[3];
+#ifndef SALONEINSTLIB
 		double cct, vct, vdt;
 		double cct_de, vct_de, vdt_de;
+#endif /* !SALONEINSTLIB */
 		int ch = '0';		/* Character */
 		int sufwa = 0;		/* Setup for FWA compensation */
 		int dofwa = 0;		/* Do FWA compensation */
@@ -965,9 +1104,13 @@ int main(int argc, char *argv[])
 			}
 
 			printf("\nThere are saved spot readings in the instrument.\n");
+#ifndef SALONEINSTLIB
 			printf("Hit [A-Z] to use reading for white and setup FWA compensation (keyed to letter)\n");
 			printf("[a-z] to use reading for FWA compensated value from keyed reference\n");
 			printf("'r' to set reference, 's' to save spectrum,\n");
+#else /* SALONEINSTLIB */
+			printf("Hit 'r' to set reference\n");
+#endif /* SALONEINSTLIB */
 			printf("Hit ESC or Q to exit, N to not read saved reading,\n");
 			printf("any other key to use reading: ");
 			fflush(stdout);
@@ -1051,7 +1194,11 @@ int main(int argc, char *argv[])
 
 			if (ambient == 2) {
 				printf("\nConfigure for ambient, press and hold button, trigger flash then release button,\n");
+#ifndef SALONEINSTLIB
 				printf("or hit 'r' to set reference, 's' to save spectrum,\n");
+#else /* SALONEINSTLIB */
+				printf("or hit 'r' to set reference\n");
+#endif /* SALONEINSTLIB */
 				printf("'h' to toggle high res., 'k' to do a calibration\n");
 
 			} else {
@@ -1081,9 +1228,13 @@ int main(int argc, char *argv[])
 						printf("\nPlace instrument on spot to be measured,\n");
 					}
 				}
+#ifndef SALONEINSTLIB
 				printf("and hit [A-Z] to read white and setup FWA compensation (keyed to letter)\n");
 				printf("[a-z] to read and make FWA compensated reading from keyed reference\n");
 				printf("'r' to set reference, 's' to save spectrum,\n");
+#else /* SALONEINSTLIB */
+				printf("Hit 'r' to set reference\n");
+#endif /* SALONEINSTLIB */
 				printf("'h' to toggle high res., 'k' to do a calibration\n");
 			}
 			if (uswitch)
@@ -1293,6 +1444,7 @@ int main(int argc, char *argv[])
 			--ix;
 			continue;
 		}
+#ifndef SALONEINSTLIB
 		if (ch == 'S' || ch == 's') {	/* Save last spectral into file */
 			if (sp.spec_n > 0) {
 				char buf[500];
@@ -1311,6 +1463,7 @@ int main(int argc, char *argv[])
 			--ix;
 			continue;
 		}
+#endif /* !SALONEINSTLIB */
 		if (ch == 'K' || ch == 'k') {	/* Do a calibration */
 			inst_code ev;
 
@@ -1352,6 +1505,7 @@ int main(int argc, char *argv[])
 			continue;
 		}
 
+#ifndef SALONEINSTLIB
 		if (ch >= 'A' && ch <= 'Z') {
 			printf("\nMeasured media to setup FWA compensation '%c'\n",ch);
 			sufwa = 1;
@@ -1395,6 +1549,7 @@ int main(int argc, char *argv[])
 		if (sufwa) {
 			error ("FWA compensation not supported in this version");
 		}
+#endif /* !SALONEINSTLIB */
 
 		/* Print and/or plot out spectrum, */
 		/* even if it's an FWA setup */
@@ -1418,6 +1573,7 @@ int main(int argc, char *argv[])
 				printf("%s%8.3f",j > 0 ? ", " : "", tsp.spec[j]);
 			printf("\n");
 
+#ifndef SALONEINSTLIB
 			/* Plot the spectrum */
 			if (pspec == 2) {
 				double xx[XSPECT_MAX_BANDS];
@@ -1469,6 +1625,7 @@ int main(int argc, char *argv[])
 				do_plot_x(xx, yy, rLab[0] >= -1.0 ? yr : NULL, NULL, nn, 1,
 				          xmin, xmax, ymin, ymax, 2.0);
 			}
+#endif /* !SALONEINSTLIB */
 		}
 
 		if (sufwa == 0) {		/* Not setting up fwa, so show reading */
@@ -1512,6 +1669,7 @@ int main(int argc, char *argv[])
 			/* XYZ is 0 .. 100 for reflective/transmissive, and absolute for emissibe here */
 			/* XYZ is 0 .. 1 for reflective/transmissive, and absolute for emissibe here */
 
+#ifndef SALONEINSTLIB
 			/* Compute color temperatures */
 			if (ambient || doCCT) {
 				icmXYZNumber wp;
@@ -1523,7 +1681,7 @@ int main(int argc, char *argv[])
 				nxyz[1] = XYZ[1] / XYZ[1];
 
 				/* Compute CCT */
-				if ((cct = icx_XYZ2ill_ct(axyz, icxIT_Ptemp, icxOT_CIE_1931_2, NULL, XYZ, NULL, 0)) < 0)
+				if ((cct = icx_XYZ2ill_ct(axyz, icxIT_Ptemp, observ, NULL, XYZ, NULL, 0)) < 0)
 					error ("Got bad cct\n");
 				axyz[0] /= axyz[1];
 				axyz[2] /= axyz[1];
@@ -1534,7 +1692,7 @@ int main(int argc, char *argv[])
 				cct_de = icmLabDE(lab, alab);
 			
 				/* Compute VCT */
-				if ((vct = icx_XYZ2ill_ct(axyz, icxIT_Ptemp, icxOT_CIE_1931_2, NULL, XYZ, NULL, 1)) < 0)
+				if ((vct = icx_XYZ2ill_ct(axyz, icxIT_Ptemp, observ, NULL, XYZ, NULL, 1)) < 0)
 					error ("Got bad vct\n");
 				axyz[0] /= axyz[1];
 				axyz[2] /= axyz[1];
@@ -1545,7 +1703,7 @@ int main(int argc, char *argv[])
 				vct_de = icmLabDE(lab, alab);
 
 				/* Compute VDT */
-				if ((vdt = icx_XYZ2ill_ct(axyz, icxIT_Dtemp, icxOT_CIE_1931_2, NULL, XYZ, NULL, 1)) < 0)
+				if ((vdt = icx_XYZ2ill_ct(axyz, icxIT_Dtemp, observ, NULL, XYZ, NULL, 1)) < 0)
 					error ("Got bad vct\n");
 				axyz[0] /= axyz[1];
 				axyz[2] /= axyz[1];
@@ -1562,6 +1720,19 @@ int main(int argc, char *argv[])
 			/* Compute Yxy from XYZ */
 			icmXYZ2Yxy(Yxy, XYZ);
 
+			/* Compute LCh from Lab */
+			icmLab2LCh(LCh, Lab);
+
+#else /* SALONEINSTLIB */
+			/* Compute D50 Lab from XYZ */
+			XYZ2Lab(Lab, XYZ);
+
+			/* Compute Yxy from XYZ */
+			XYZ2Yxy(Yxy, XYZ);
+
+			/* Compute LCh from Lab */
+			Lab2LCh(LCh, Lab);
+#endif /* SALONEINSTLIB */
 
 			if (displ > 1 || proj > 1) {
 				if (wXYZ[0] < 0.0) {
@@ -1580,6 +1751,7 @@ int main(int argc, char *argv[])
 					XYZ[1] = 100.0 * XYZ[1] / wXYZ[1];
 					XYZ[2] = 100.0 * XYZ[2] / wXYZ[1];
 				} 
+#ifndef SALONEINSTLIB
 				  else {
 
 					/* Normalize to white and scale to 0..100 */
@@ -1593,6 +1765,27 @@ int main(int argc, char *argv[])
 
 				/* recompute Yxy from XYZ */
 				icmXYZ2Yxy(Yxy, XYZ);
+
+				/* recompute LCh from Lab */
+				icmLab2LCh(LCh, Lab);
+#else /* SALONEINSTLIB */
+				  else {
+
+					/* Normalize to white */
+					XYZ[0] = XYZ[0] * D50_X_100 / wXYZ[0];
+					XYZ[1] = XYZ[1] * D50_Y_100 / wXYZ[1];
+					XYZ[2] = XYZ[2] * D50_Z_100 / wXYZ[2];
+				}
+
+				/* recompute Lab */
+				XYZ2Lab(Lab, XYZ);
+
+				/* recompute Yxy from XYZ */
+				XYZ2Yxy(Yxy, XYZ);
+
+				/* recompute LCh from Lab */
+				Lab2LCh(LCh, Lab);
+#endif /* SALONEINSTLIB */
 			}
 
 			if (ambient && (cap & inst_emis_ambient_mono)) {
@@ -1602,6 +1795,10 @@ int main(int argc, char *argv[])
 					/* Print out the XYZ and Yxy */
 					printf("\n Result is XYZ: %f %f %f, Yxy: %f %f %f\n",
 					XYZ[0], XYZ[1], XYZ[2], Yxy[0], Yxy[1], Yxy[2]);
+				} else if (doLCh) {
+					/* Print out the XYZ and LCh */
+					printf("\n Result is XYZ: %f %f %f, LCh: %f %f %f\n",
+					XYZ[0], XYZ[1], XYZ[2], LCh[0], LCh[1], LCh[2]);
 				} else {
 					/* Print out the XYZ and Lab */
 					printf("\n Result is XYZ: %f %f %f, D50 Lab: %f %f %f\n",
@@ -1610,9 +1807,15 @@ int main(int argc, char *argv[])
 			}
 
 			if (rLab[0] >= -1.0) {
+#ifndef SALONEINSTLIB
 				printf(" Delta E to reference is %f %f %f (%f, CIE94 %f)\n",
 				Lab[0] - rLab[0], Lab[1] - rLab[1], Lab[2] - rLab[2],
 				icmLabDE(Lab, rLab), icmCIE94(Lab, rLab));
+#else
+				printf(" Delta E to reference is %f %f %f (%f)\n",
+				Lab[0] - rLab[0], Lab[1] - rLab[1], Lab[2] - rLab[2],
+				LabDE(Lab, rLab));
+#endif
 			}
 			if (ambient) {
 				if (ambient == 2)
@@ -1625,6 +1828,7 @@ int main(int argc, char *argv[])
 							       3.141592658 * XYZ[1],
 						           log(3.141592658 * XYZ[1]/2.5)/log(2.0));
 				} else {
+#ifndef SALONEINSTLIB
 					printf(" Ambient = %.1f Lux%s, CCT = %.0fK (Delta E %f)\n",
 					       3.1415926 * XYZ[1], ambient == 2 ? "-Seconds" : "",
 					       cct, cct_de);
@@ -1634,18 +1838,26 @@ int main(int argc, char *argv[])
 						           log(3.141592658 * XYZ[1]/2.5)/log(2.0));
 					printf(" Closest Planckian temperature = %.0fK (Delta E %f)\n",vct, vct_de);
 					printf(" Closest Daylight temperature  = %.0fK (Delta E %f)\n",vdt, vdt_de);
+#else /* SALONEINSTLIB */
+					printf(" Ambient = %.1f Lux%s\n",
+					       3.1415926 * XYZ[1], ambient == 2 ? "-Seconds" : "");
+#endif /* SALONEINSTLIB */
 				}
+#ifndef SALONEINSTLIB
 			} else if (doCCT) {
 				printf("                           CCT = %.0fK (Delta E %f)\n",cct, cct_de);
 				printf(" Closest Planckian temperature = %.0fK (Delta E %f)\n",vct, vct_de);
 				printf(" Closest Daylight temperature  = %.0fK (Delta E %f)\n",vdt, vdt_de);
+#endif
 			}
+#ifndef SALONEINSTLIB
 			if (val.sp.spec_n > 0 && (ambient || doCCT)) {
 				int invalid = 0;
 				double cri;
 				cri = icx_CIE1995_CRI(&invalid, &sp);
 				printf(" Color Rendering Index (Ra) = %.1f%s\n",cri,invalid ? " (Invalid)" : "");
 			}
+#endif
 
 			/* Save reading to the log file */
 			if (fp != NULL) {
