@@ -12,8 +12,8 @@
  *
  * (Based initially on i1disp.c)
  *
- * This material is licenced under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 :-
- * see the License.txt file for licencing details.
+ * This material is licenced under the GNU GENERAL PUBLIC LICENSE Version 2 or later :-
+ * see the License2.txt file for licencing details.
  */
 
 /*
@@ -63,9 +63,14 @@
 #include <time.h>
 #include <stdarg.h>
 #include <math.h>
+#ifndef SALONEINSTLIB
 #include "copyright.h"
 #include "aconfig.h"
 #include "numlib.h"
+#else /* SALONEINSTLIB */
+#include "sa_config.h"
+#include "numsup.h"
+#endif /* SALONEINSTLIB */
 #include "xspect.h"
 #include "insttypes.h"
 #include "icoms.h"
@@ -887,7 +892,7 @@ spyd2_GetReading_ll(
 					_mintcnt = transcnt;
 			}
 			if (p->debug >= 4)
-				printf("%d: initial senv %f from transcnt %d and intclls %d\n",k,sensv[k],transcnt,intclks);
+				fprintf(stderr,"%d: initial senv %f from transcnt %d and intclls %d\n",k,sensv[k],transcnt,intclks);
 
 #ifdef NEVER	/* This seems to make repeatability worse ??? */
 			/* If CRT and bright enough */
@@ -949,7 +954,7 @@ spyd2_GetReading_ll(
 					_mintcnt = transcnt;
 			}
 			if (p->debug >= 4)
-				printf("%d: initial senv %f from transcnt %d and intclls %d\n",k,sensv[k],transcnt,intclks);
+				fprintf(stderr,"%d: initial senv %f from transcnt %d and intclls %d\n",k,sensv[k],transcnt,intclks);
 		}
 	}
 
@@ -1518,7 +1523,7 @@ spyd2_GetReading(
 											/* (First table is not noticably different */
 											/*  but can have scaled values.) */
 	if (p->debug >= 4)
-		printf("Using cal table %d\n",table);
+		fprintf(stderr,"Using cal table %d\n",table);
 
 	for (k = 0; k < 8; k++) 	/* Zero weighted average */
 		a_sensv[k] = a_w[k] = 0.0;
@@ -1739,13 +1744,13 @@ spyd2_read_all_regs(
 	if ((ev = spyd2_rd_ee_ushort(p, &p->hwver, 5)) != inst_ok)
 		return ev;
 
-	if (p->debug >= 4) printf("hwver = 0x%x\n",p->hwver);
+	if (p->debug >= 1) fprintf(stderr,"hwver = 0x%x\n",p->hwver);
 
 	/* Serial number */
 	if ((ev = spyd2_readEEProm(p, (unsigned char *)p->serno, 8, 8)) != inst_ok)
 		return ev;
 	p->serno[8] = '\000';
-	if (p->debug >= 4) printf("serno = '%s'\n",p->serno);
+	if (p->debug >= 4) fprintf(stderr,"serno = '%s'\n",p->serno);
 
 	/* Spyde2: CRT calibration values */
 	/* Spyde3: Unknown calibration values */
@@ -1771,10 +1776,10 @@ spyd2_read_all_regs(
 			}
 		}
 		avgmag /= (double)(i);
-		if (p->debug >= 4) printf("Cal_A avgmag = %f\n",avgmag);
+		if (p->debug >= 4) fprintf(stderr,"Cal_A avgmag = %f\n",avgmag);
 
 		if (avgmag < 0.05) {
-			if (p->debug >= 4) printf("Scaling Cal_A by 16\n");
+			if (p->debug >= 4) fprintf(stderr,"Scaling Cal_A by 16\n");
 			for (j = 0; j < 3; j++) {
 				for (k = 0; k < 9; k++) {
 					p->cal_A[0][j][k] *= 16.0;
@@ -1812,27 +1817,27 @@ spyd2_read_all_regs(
 		int i, j, k;
 
 		
-		printf("Cal_A:\n");
+		fprintf(stderr,"Cal_A:\n");
 		for (i = 0; i < 2;i++) {
 			for (j = 0; j < 3; j++) {
 				for (k = 0; k < 9; k++) {
-					printf("Cal_A [%d][%d][%d] = %f\n",i,j,k,p->cal_A[i][j][k]);
+					fprintf(stderr,"Cal_A [%d][%d][%d] = %f\n",i,j,k,p->cal_A[i][j][k]);
 				}
 			}
 		}
-		printf("\nCal_B:\n");
+		fprintf(stderr,"\nCal_B:\n");
 		for (i = 0; i < 2;i++) {
 			for (j = 0; j < 3; j++) {
 				for (k = 0; k < 9; k++) {
-					printf("Cal_B [%d][%d][%d] = %f\n",i,j,k,p->cal_B[i][j][k]);
+					fprintf(stderr,"Cal_B [%d][%d][%d] = %f\n",i,j,k,p->cal_B[i][j][k]);
 				}
 			}
 		}
-		printf("\nCal_F:\n");
+		fprintf(stderr,"\nCal_F:\n");
 		for (i = 0; i < 7;i++) {
-			printf("Cal_F [%d] = %f\n",i,p->cal_F[i]);
+			fprintf(stderr,"Cal_F [%d] = %f\n",i,p->cal_F[i]);
 		}
-		printf("\n");
+		fprintf(stderr,"\n");
 	}
 
 	if (p->debug) fprintf(stderr,"spyd2: all EEProm read OK\n");
@@ -1921,6 +1926,8 @@ spyd2_init_coms(inst *pp, int port, baud_rate br, flow_control fc, double tout)
 
 	/* On MSWindows the Spyder 3 doesn't work reliably unless each */
 	/* read is preceeded by a reset endpoint. */
+	/* (!!! This needs checking to see if it's true. */
+	/*  Should switch back to libusb0.sys and re-test.)
 	/* (and Spyder 2 hangs if a reset ep is done.) */
 	/* The spyder 2 doesn't work well with the winusb driver either, */
 	/* it needs icomuf_resetep_before_read to work at all, and */
@@ -1948,7 +1955,7 @@ spyd2_init_coms(inst *pp, int port, baud_rate br, flow_control fc, double tout)
 
 	/* Set config, interface, write end point, read end point */
 	/* ("serial" end points aren't used - the spyd2lay uses USB control messages) */
-	p->icom->set_usb_port(p->icom, port, 1, 0x00, 0x00, usbflags, 0); 
+	p->icom->set_usb_port(p->icom, port, 1, 0x00, 0x00, usbflags, 0, NULL); 
 
 	if (p->debug) fprintf(stderr,"spyd2: init coms has suceeded\n");
 
@@ -2326,6 +2333,7 @@ inst_capability spyd2_capabilities(inst *pp) {
 
 	if (p->itype == instSpyder3
 	 && (p->hwver & 0x8) == 0) {	/* Not Spyder3Express */
+// ~~~999 this doesn't seem right ????
 		rv |= inst_emis_ambient;
 		rv |= inst_emis_ambient_mono;
 	}
