@@ -11,8 +11,8 @@
  * Copyright 2006 - 2007, Graeme W. Gill
  * All rights reserved.
  *
- * This material is licenced under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 :-
- * see the License.txt file for licencing details.
+ * This material is licenced under the GNU GENERAL PUBLIC LICENSE Version 2 or later :-
+ * see the License2.txt file for licencing details.
  */
 
 /* This program enables the Spyder 2 instrument for Argyll, */
@@ -36,12 +36,19 @@
 #include <windows.h>
 #endif
 #ifdef UNIX
+#include <unistd.h>
 #include <sys/param.h>
 #include <sys/mount.h>
 #endif /* UNIX */
+#ifndef SALONEINSTLIB
 #include "copyright.h"
 #include "aconfig.h"
 #include "numlib.h"
+#else /* SALONEINSTLIB */
+#include <fcntl.h>
+#include "sa_config.h"
+#include "numsup.h"
+#endif /* SALONEINSTLIB */
 #include "xdg_bds.h"
 
 /* --------------------------------------------------------- */
@@ -195,7 +202,7 @@ int write_output(unsigned char *buf, unsigned int len) {
 		(g_va->dsize + len), g_va->maxdsize);
 		return 1;
 	}
-	memcpy(g_va->dbuf + g_va->dsize, buf, len);
+	memmove(g_va->dbuf + g_va->dsize, buf, len);
 	g_va->dsize += len;
 	return 0;
 }
@@ -221,6 +228,9 @@ archive *new_arch(char *name, int verb) {
 	p->del = del_arch;
 
 	/* Open up the file for reading */
+#if !defined(O_CREAT) && !defined(_O_CREAT)
+# error "Need to #include fcntl.h!"
+#endif
 #if defined(O_BINARY) || defined(_O_BINARY)
 	if ((ifp = fopen(name,"rb")) == NULL)
 #else
@@ -394,6 +404,9 @@ int patch_exe(char *name, unsigned char *fbuf, unsigned int fsize, int verb) {
 		printf("About to patch executable '%s'\n",name);
 
 	/* open executable file */
+#if !defined(O_CREAT) && !defined(_O_CREAT)
+# error "Need to #include fcntl.h!"
+#endif
 #if defined(O_BINARY) || defined(_O_BINARY)
 	if ((fp = fopen(name,"r+b")) == NULL)
 #else
@@ -522,6 +535,9 @@ int write_bin(char *name, unsigned char *fbuf, unsigned int fsize, int verb) {
 		printf("About to write binary '%s'\n",name);
 
 	/* open executable file */
+#if !defined(O_CREAT) && !defined(_O_CREAT)
+# error "Need to #include fcntl.h!"
+#endif
 #if defined(O_BINARY) || defined(_O_BINARY)
 	if ((fp = fopen(name,"wb")) == NULL)
 #else
@@ -554,7 +570,7 @@ int write_bin(char *name, unsigned char *fbuf, unsigned int fsize, int verb) {
 
 void usage(void) {
 	fprintf(stderr,"Transfer Spyder 2 PLD pattern from file, Version %s\n",ARGYLL_VERSION_STR);
-	fprintf(stderr,"Author: Graeme W. Gill, licensed under the GPL Version 3\n");
+	fprintf(stderr,"Author: Graeme W. Gill, licensed under the GPL Version 2 or later\n");
 	fprintf(stderr,"usage: Transfer [-v] infile\n");
 	fprintf(stderr," -v              Verbose\n");
 	fprintf(stderr," -S d            Specify the install scope u = user (def.), l = local system]\n");
@@ -797,8 +813,11 @@ main(int argc, char *argv[]) {
 			char *vols[] = {		/* Typical volumes the CDROM could be mounted under */
 				"/media/ColorVision",
 				"/mnt/cdrom",
+				"/mnt/cdrecorder",
 				"/media/cdrom",
+				"/media/cdrecorder",
 				"/cdrom",
+				"/cdrecorder",
 				""
 			};
 
