@@ -13,8 +13,8 @@
  * Copyright 2005 - 2007 Graeme W. Gill
  * All rights reserved.
  *
- * This material is licenced under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 :-
- * see the License.txt file for licencing details.
+ * This material is licenced under the GNU GENERAL PUBLIC LICENSE Version 2 or later :-
+ * see the License2.txt file for licencing details.
  *
  * This is an alternative driver to spm/gretag.
  */
@@ -42,9 +42,11 @@
 #include <ctype.h>
 #include <string.h>
 #include <time.h>
+#ifndef SALONEINSTLIB
 #include "copyright.h"
-#include "config.h"
+#include "aconfig.h"
 #include "numlib.h"
+#endif /* !SALONEINSTLIB */
 #include "xspect.h"
 #include "insttypes.h"
 #include "icoms.h"
