@@ -1,23 +1,25 @@
 
+ /* Unix serial I/O class */
+
 /* 
  * Argyll Color Correction System
  *
- * Linux serial I/O class
- *
  * Author: Graeme W. Gill
  * Date:   18/11/2000
  *
- * Copyright 1997 - 2007 Graeme W. Gill
+ * Copyright 1997 - 2010 Graeme W. Gill
  * All rights reserved.
  *
- * This material is licenced under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 :-
- * see the License.txt file for licencing details.
+ * This material is licenced under the GNU GENERAL PUBLIC LICENSE Version 2 or later :-
+ * see the License2.txt file for licencing details.
  */
 
 /*
     TTBD:
  */
 
+#ifdef UNIX
+
 #include <sys/types.h>		/* Include sys/select.h ? */
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -26,12 +28,15 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <dirent.h>
 #include <time.h>
 #include <errno.h>
 #include <string.h>
+#ifndef SALONEINSTLIB
 #include "copyright.h"
-#include "config.h"
-#include "numlib.h"
+#include "aconfig.h"
+#endif
+#include "numsup.h"
 #include "xspect.h"
 #include "insttypes.h"
 #include "icoms.h"
@@ -112,6 +117,7 @@ icoms *p
 	hid_get_paths(p);
 	usbend = p->npaths;
 
+#ifdef ENABLE_SERIAL
 #ifdef __APPLE__
 	/* Search the OSX registry for serial ports */
 	{
@@ -226,6 +232,16 @@ icoms *p
 		                 Linux           /dev/ttyUSB0, /dev/ttyUSB1
 	*/
 
+	/*
+		"Most program set a lock in /var/lock/LCK..tty<XX> on Linux ?
+		<http://sunsite.ualberta.ca/LDP/LDP/nag2/x-087-2-serial.devices.html>
+		<http://docs.freebsd.org/info/uucp/uucp.info.UUCP_Lock_Files.html>
+
+		We should really use the lock files to avoid treading on
+		other programs toes. We assume at the moment that the user
+		only picks a serial port with an instrument on it.
+	*/
+
 	/* Search for devices that match the pattern /dev/ttyS[0-9]* and /dev/ttyUSB* */
 	{
 		DIR *dd;
@@ -246,9 +262,16 @@ icoms *p
 				break;
 
 			if (!(
+#ifdef __FreeBSD__
+			   /* This should match uart & USB devs. */
+				( strncmp (de->d_name, "cua", 3) == 0
+				&& strlen (de->d_name) < 7)
+#else
+				/* Presumably Linux.. */
 			    (   strncmp(de->d_name, "ttyS", 4) == 0
 			     && de->d_name[4] >= '0' && de->d_name[4] <= '9')
 			 || (   strncmp(de->d_name, "ttyUSB", 5) == 0)
+#endif
 			))
 				continue;
 
@@ -259,11 +282,23 @@ icoms *p
 			strcpy(dpath, dirn);
 			strcat(dpath, de->d_name);
 
-			if ((fd = open(dpath, O_RDWR | O_NOCTTY )) < 0) {
+			if ((fd = open(dpath, O_RDWR | O_NOCTTY | O_NONBLOCK )) < 0) {
 				if (p->debug) fprintf(errout,"failed to open serial \"%s\"\n",dpath);
 				free(dpath);
 				continue;
 			}
+			/* On linux we could do a 
+				struct serial_struct serinfo;
+
+				serinfo.reserved_char[0] = 0;
+
+				if (ioctl(fd, TIOCGSERIAL, &serinfo) < 0
+					|| serinfo.type == PORT_UNKNOWN) {
+					free(dpath);
+					continue;
+				}
+
+			 */
 			close(fd);
 			if (p->debug) fprintf(errout,"managed to open serial \"%s\"\n",dpath);
 
@@ -297,21 +332,21 @@ icoms *p
 			p->paths[p->npaths] = NULL;
 		}
 		closedir(dd);
+	}
+#endif /* ! __APPLE__ */
+#endif /* ENABLE_SERIAL */
 
-		/* Sort the /dev keys so people don't get confused... */
-		for (i = usbend; i < (p->npaths-1); i++) {
-			for (j = i+1; j < p->npaths; j++) {
-				if (strcmp(p->paths[i]->path, p->paths[j]->path) > 0) {
-					icompath *tt = p->paths[i];
-					p->paths[i] = p->paths[j];
-					p->paths[j] = tt;
-				}
+	/* Sort the /dev keys so people don't get confused... */
+	for (i = usbend; i < (p->npaths-1); i++) {
+		for (j = i+1; j < p->npaths; j++) {
+			if (strcmp(p->paths[i]->path, p->paths[j]->path) > 0) {
+				icompath *tt = p->paths[i];
+				p->paths[i] = p->paths[j];
+				p->paths[j] = tt;
 			}
 		}
 	}
 
-#endif /* ! __APPLE__ */
-
 	return p->paths;
 }
 
@@ -829,7 +864,7 @@ icoms_del(icoms *p) {
 }
 
 /* Constructor */
-extern icoms *new_icoms() {
+icoms *new_icoms() {
 	icoms *p;
 	if ((p = (icoms *)calloc(sizeof(icoms), 1)) == NULL)
 		error("icoms: malloc failed!");
@@ -859,3 +894,4 @@ extern icoms *new_icoms() {
 	return p;
 }
 
+#endif /* UNIX */
