@@ -18,8 +18,15 @@
 #include <fcntl.h>
 #include <string.h>
 #include <sys/types.h>
+#ifndef SALONEINSTLIB
+#include "copyright.h"
 #include "aconfig.h"
 #include "numlib.h"
+#else /* SALONEINSTLIB */
+#include <fcntl.h>
+#include "sa_config.h"
+#include "numsup.h"
+#endif /* SALONEINSTLIB */
 
 #undef DEBUG
 
@@ -544,7 +551,7 @@ int bl, bd;             /* number of bits decoded by tl[] and td[] */
 #if !defined(NOMEMCPY) && !defined(DEBUG)
         if (w - d >= e)         /* (this test assumes unsigned comparison) */
         {
-          memcpy(slide + w, slide + d, e);
+          memmove(slide + w, slide + d, e);
           w += e;
           d += e;
         }
