@@ -39,7 +39,6 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <time.h>
 #include "icc.h"
 #include "jcnf.h"
 #include "ucmm.h"
@@ -173,6 +172,7 @@ ucmm_error ucmm_install_monitor_profile(
 
 	/* Locate the directories where the config file is, */
 	/* and where we should copy the profile to. */
+	/* !! Should switch to xdg_bds() function !! */
 
 	cnl = strlen(config_file);
 	pnl = strlen(profile_dir) + 1 + strlen(profile);	/* Including "/" */
@@ -183,7 +183,7 @@ ucmm_error ucmm_install_monitor_profile(
 			tt = "/etc/xdg";
 		if ((conf_name = malloc(strlen(tt) + cnl + 2)) == NULL)
 			return ucmm_resource;
-		memcpy(conf_name, tt, strlen(tt)+1);
+		memmove(conf_name, tt, strlen(tt)+1);
 
 		if ((tt = getenv("XDG_DATA_DIRS")) == NULL)
 			tt = "/usr/local/share/:/usr/share/";
@@ -191,19 +191,19 @@ ucmm_error ucmm_install_monitor_profile(
 			free(conf_name);
 			return ucmm_resource;
 		}
-		memcpy(data_name, tt, strlen(tt)+1);
+		memmove(data_name, tt, strlen(tt)+1);
 	} else {
 		char *tt, *home;
 		if ((tt = getenv("XDG_CONFIG_HOME")) != NULL) {
 			if ((conf_name = malloc(strlen(tt) + cnl + 2)) == NULL)
 				return ucmm_resource;
-			memcpy(conf_name, tt, strlen(tt)+1);
+			memmove(conf_name, tt, strlen(tt)+1);
 		} else {
 			if ((home = getenv("HOME")) == NULL)
 				return ucmm_no_home;
 			if ((conf_name = malloc(strlen(home) + strlen(".config") + cnl +3)) == NULL)
 				return ucmm_resource;
-			memcpy(conf_name, home, strlen(home)+1);
+			memmove(conf_name, home, strlen(home)+1);
 			if (strlen(conf_name) > 0 && conf_name[strlen(conf_name)-1] != '/')
 				strcat(conf_name, "/");
 			strcat(conf_name, ".config");
@@ -213,9 +213,8 @@ ucmm_error ucmm_install_monitor_profile(
 				free(conf_name);
 				return ucmm_resource;
 			}
-			memcpy(data_name, tt, strlen(tt)+1);
+			memmove(data_name, tt, strlen(tt)+1);
 		} else {
-			data_name = ".local/share";
 			if ((home = getenv("HOME")) == NULL) {
 				free(conf_name);
 				return ucmm_no_home;
@@ -224,7 +223,7 @@ ucmm_error ucmm_install_monitor_profile(
 				free(conf_name);
 				return ucmm_resource;
 			}
-			memcpy(data_name, home, strlen(home)+1);
+			memmove(data_name, home, strlen(home)+1);
 			if (strlen(data_name) > 0 && data_name[strlen(data_name)-1] != '/')
 				strcat(data_name, "/");
 			strcat(data_name, ".local/share");
@@ -510,6 +509,7 @@ ucmm_error ucmm_uninstall_monitor_profile(
 
 	/* Locate the directories where the config file is, */
 	/* and where the profile should be too. */
+	/* !! Should switch to xdg_bds() function !! */
 
 	cnl = strlen(config_file);
 	pnl = strlen(profile_dir) + 1;
@@ -522,7 +522,7 @@ ucmm_error ucmm_uninstall_monitor_profile(
 			tt = "/etc/xdg";
 		if ((conf_name = malloc(strlen(tt) + cnl + 2)) == NULL)
 			return ucmm_resource;
-		memcpy(conf_name, tt, strlen(tt)+1);
+		memmove(conf_name, tt, strlen(tt)+1);
 
 		if ((tt = getenv("XDG_DATA_DIRS")) == NULL)
 			tt = "/usr/local/share/:/usr/share/";
@@ -530,19 +530,19 @@ ucmm_error ucmm_uninstall_monitor_profile(
 			free(conf_name);
 			return ucmm_resource;
 		}
-		memcpy(data_name, tt, strlen(tt)+1);
+		memmove(data_name, tt, strlen(tt)+1);
 	} else {
 		char *tt, *home;
 		if ((tt = getenv("XDG_CONFIG_HOME")) != NULL) {
 			if ((conf_name = malloc(strlen(tt) + cnl + 2)) == NULL)
 				return ucmm_resource;
-			memcpy(conf_name, tt, strlen(tt)+1);
+			memmove(conf_name, tt, strlen(tt)+1);
 		} else {
 			if ((home = getenv("HOME")) == NULL)
 				return ucmm_no_home;
 			if ((conf_name = malloc(strlen(home) + strlen(".config") + cnl +3)) == NULL)
 				return ucmm_resource;
-			memcpy(conf_name, home, strlen(home)+1);
+			memmove(conf_name, home, strlen(home)+1);
 			if (strlen(conf_name) > 0 && conf_name[strlen(conf_name)-1] != '/')
 				strcat(conf_name, "/");
 			strcat(conf_name, ".config");
@@ -552,7 +552,7 @@ ucmm_error ucmm_uninstall_monitor_profile(
 				free(conf_name);
 				return ucmm_resource;
 			}
-			memcpy(data_name, tt, strlen(tt)+1);
+			memmove(data_name, tt, strlen(tt)+1);
 		} else {
 			data_name = ".local/share";
 			if ((home = getenv("HOME")) == NULL) {
@@ -563,7 +563,7 @@ ucmm_error ucmm_uninstall_monitor_profile(
 				free(conf_name);
 				return ucmm_resource;
 			}
-			memcpy(data_name, home, strlen(home)+1);
+			memmove(data_name, home, strlen(home)+1);
 			if (strlen(data_name) > 0 && data_name[strlen(data_name)-1] != '/')
 				strcat(data_name, "/");
 			strcat(data_name, ".local/share");
@@ -830,6 +830,7 @@ ucmm_error ucmm_get_monitor_profile(
 
 	/* Locate the directories where the config file is, */
 	/* and where the profile should be too. */
+	/* !! Should switch to xdg_bds() function !! */
 
 	cnl = strlen(config_file);
 
@@ -843,7 +844,7 @@ ucmm_error ucmm_get_monitor_profile(
 				tt = "/etc/xdg";
 			if ((conf_name = malloc(strlen(tt) + cnl + 2)) == NULL)
 				return ucmm_resource;
-			memcpy(conf_name, tt, strlen(tt)+1);
+			memmove(conf_name, tt, strlen(tt)+1);
 
 		} else {					/* User */
 			char *tt, *home;
@@ -851,13 +852,13 @@ ucmm_error ucmm_get_monitor_profile(
 			if ((tt = getenv("XDG_CONFIG_HOME")) != NULL) {
 				if ((conf_name = malloc(strlen(tt) + cnl + 2)) == NULL)
 					return ucmm_resource;
-				memcpy(conf_name, tt, strlen(tt)+1);
+				memmove(conf_name, tt, strlen(tt)+1);
 			} else {
 				if ((home = getenv("HOME")) == NULL)
 					return ucmm_no_home;
 				if ((conf_name = malloc(strlen(home) + strlen(".config") + cnl +3)) == NULL)
 					return ucmm_resource;
-				memcpy(conf_name, home, strlen(home)+1);
+				memmove(conf_name, home, strlen(home)+1);
 				if (strlen(conf_name) > 0 && conf_name[strlen(conf_name)-1] != '/')
 					strcat(conf_name, "/");
 				strcat(conf_name, ".config");
