@@ -32,7 +32,7 @@
 #include <string.h>
 #include <math.h>
 #include "copyright.h"
-#include "config.h"
+#include "aconfig.h"
 #include "numlib.h"
 #include "tiffio.h"
 #include "icc.h"
@@ -52,7 +52,7 @@ void del_filter();
 void usage(void) {
 	int i;
 	fprintf(stderr,"Create VRML image of the gamut surface of a TIFF, Version %s\n",ARGYLL_VERSION_STR);
-	fprintf(stderr,"Author: Graeme W. Gill, licensed under the GPL Version 3\n");
+	fprintf(stderr,"Author: Graeme W. Gill, licensed under the AGPL Version 3\n");
 	fprintf(stderr,"usage: tiffgamut [-v level] [profile.icm | embedded.tif] infile1.tif [infile2.tif ...] \n");
 	fprintf(stderr," -v            Verbose\n");
 	fprintf(stderr," -d sres       Surface resolution details 1.0 - 50.0\n");
@@ -75,12 +75,13 @@ void usage(void) {
 
 		fprintf(stderr,"           %s\n",vc.desc);
 	}
-	fprintf(stderr,"         s:surround    a = average, m = dim, d = dark,\n");
+	fprintf(stderr,"         s:surround    n = auto, a = average, m = dim, d = dark,\n");
 	fprintf(stderr,"                       c = transparency (default average)\n");
 	fprintf(stderr,"         w:X:Y:Z       Adapted white point as XYZ (default media white)\n");
 	fprintf(stderr,"         w:x:y         Adapted white point as x, y\n");
 	fprintf(stderr,"         a:adaptation  Adaptation luminance in cd.m^2 (default 50.0)\n");
 	fprintf(stderr,"         b:background  Background %% of image luminance (default 20)\n");
+	fprintf(stderr,"         l:scenewhite  Scene white in cd.m^2 if surround = auto (default 250)\n");
 	fprintf(stderr,"         f:flare       Flare light %% of image luminance (default 1)\n");
 	fprintf(stderr,"         f:X:Y:Z       Flare color as XYZ (default media white)\n");
 	fprintf(stderr,"         f:x:y         Flare color as x, y\n");
@@ -296,6 +297,7 @@ main(int argc, char *argv[]) {
 	double vc_wxy[2] = {-1.0, -1.0};		/* Adapted white override in x,y */
 	double vc_a = -1.0;			/* Adapted luminance */
 	double vc_b = -1.0;			/* Background % overid */
+	double vc_l = -1.0;			/* Scene luminance override */
 	double vc_f = -1.0;			/* Flare % overid */
 	double vc_fXYZ[3] = {-1.0, -1.0, -1.0};	/* Flare color override in XYZ */
 	double vc_fxy[2] = {-1.0, -1.0};		/* Flare color override in x,y */
@@ -438,7 +440,9 @@ main(int argc, char *argv[]) {
 				} else if (na[0] == 's' || na[0] == 'S') {
 					if (na[1] != ':')
 						usage();
-					if (na[2] == 'a' || na[2] == 'A') {
+					if (na[2] == 'n' || na[2] == 'N') {
+						vc_s = vc_none;		/* Automatic */
+					} else if (na[2] == 'a' || na[2] == 'A') {
 						vc_s = vc_average;
 					} else if (na[2] == 'm' || na[2] == 'M') {
 						vc_s = vc_dim;
@@ -464,6 +468,10 @@ main(int argc, char *argv[]) {
 					if (na[1] != ':')
 						usage();
 					vc_b = atof(na+2);
+				} else if (na[0] == 'l' || na[0] == 'L') {
+					if (na[1] != ':')
+						usage();
+					vc_l = atof(na+2);
 				} else if (na[0] == 'f' || na[0] == 'F') {
 					double x, y, z;
 					if (sscanf(na+1,":%lf:%lf:%lf",&x,&y,&z) == 3) {
@@ -601,6 +609,8 @@ main(int argc, char *argv[]) {
 			vc.La = vc_a;
 		if (vc_b >= 0.0)
 			vc.Yb = vc_b/100.0;
+		if (vc_l >= 0.0)
+			vc.Lv = vc_l;
 		if (vc_f >= 0.0)
 			vc.Yf = vc_f/100.0;
 		if (vc_fXYZ[1] > 0.0) {
@@ -678,7 +688,7 @@ main(int argc, char *argv[]) {
 			error("new_icxcam failed");
 
 		cam->set_view(cam, vc.Ev, vc.Wxyz, vc.La, vc.Yb, vc.Lv, vc.Yf, vc.Fxyz,
-		              XICC_USE_HK, XICC_NOCAMCL);
+		              XICC_USE_HK);
 	}
 
 	/* Establish the PCS range if we are filtering */
