@@ -22,6 +22,10 @@
 /*
  * TTBD:
  *
+ *		The ICX_CLIP_WB implementation could probably be smarter,
+ *      and constrain the white and black in the matrix optimizations,
+ *		as well as clipping them afterwards.
+ *
  *		Should the input profile white point determination
  *		be made a bit smarter about determining the chromaticity ?
  *
@@ -34,15 +38,24 @@
  *      Note that if curves have scale, the scale will have to be
  *      normalized back to zero by scaling the matrix before storing
  *      the result in the ICC profile.
+ *
+ *		See xlut.c for outline of future chromatic adapation approachs.
  */
 
 #define USE_CIE94_DE	/* Use CIE94 delta E measure when creating fit */
 
 /* Weights in shaper parameters, to minimise unconstrained "wiggles" */
 #define MXNORDERS 30			/* Maximum shaper harmonic orders to use */
-#define XSHAPE_MAG  5000.0		/* Overall shaper parameter magnitide */
-#define XSHAPE_BASE  0.00001	/* 0 & 1 harmonic weight */
-#define XSHAPE_HBASE 0.0001		/* 2nd and higher additional weight */
+#define XSHAPE_MAG  1.0			/* Overall shaper parameter magnitide */
+
+#define XSHAPE_OFFG			0.1		/* Default offset weights */
+#define XSHAPE_OFFS			1.0		/* Input offset weights when ord 0 is shaper */
+#define XSHAPE_HW01			0.2		/* 0 & 1 harmonic weights */
+#define XSHAPE_HBREAK	    4		/* Harmonic that has HWBR */
+#define XSHAPE_HWBR        0.8		/* Base weight of harmonics HBREAK up */
+#define XSHAPE_HWINC       0.5		/* Increase in weight for each harmonic above HWBR */
+
+#define XSHAPE_GAMTHR	   0.01		/* Input threshold for linear slope below gamma power */
 
 #undef DEBUG			/* Extra printfs */
 #undef DEBUG_PLOT		/* Plot curves */
@@ -297,7 +310,7 @@ int                   dir			/* 0 = fwd, 1 = bwd */
 			xicc_enum_viewcond(xicp, &p->vc, -1, NULL, 0, NULL);	/* Use a default */
 		p->cam = new_icxcam(cam_default);
 		p->cam->set_view(p->cam, p->vc.Ev, p->vc.Wxyz, p->vc.La, p->vc.Yb, p->vc.Lv, p->vc.Yf,
-		                 p->vc.Fxyz, XICC_USE_HK, XICC_NOCAMCL);
+		                 p->vc.Fxyz, XICC_USE_HK);
 	} else {
 		p->cam = NULL;
 	}
@@ -350,7 +363,7 @@ typedef struct {
 	double min, max;	/* device End points to linearise */
 } mxinctx;
 
-#define NPARMS (15 + 3 * MXNORDERS)
+#define NPARMS (9 + 6 + 3 * MXNORDERS)
 
 /* Context for optimising matrix */
 typedef struct {
@@ -359,7 +372,12 @@ typedef struct {
 	int isLinear;			/* NZ if no curves, fixed Gamma = 1.0 */
 	int isGamma;			/* NZ if gamma + matrix, else shaper */
 	int isShTRC;			/* NZ if shared TRC */
+	int shape0gam;			/* NZ if zero'th order shaper should be gamma function */
 	int norders;			/* Number of shaper orders */
+	int clipbw;				/* Prevent white > 1 and -ve black */
+	int clipprims;			/* Prevent primaries going -ve */
+	double smooth;			/* Shaper smoothing factor (nominal = 1.0) */ 
+	double dscale;			/* Scale device values */
 	double v[NPARMS];		/* Holder for parameters */
 	double sa[NPARMS];		/* Initial search area */
 							/* Rest are matrix : */
@@ -369,11 +387,13 @@ typedef struct {
 							/* For Gamma: */
 							/* 9, 10, 11 are gamma */
 							/* Else for shaper only: */
-							/* 9,  10, 11 are Offset */
-							/* 12, 13, 14 are 0th harmonics */
-							/* 15, 16, 17 are 1st harmonics */
-							/* 18, 19, 20 are 2nd harmonics */
+							/* 9,  10, 11 are Input Offset */
+							/* 12, 13, 14 are Output Offset */
+							/* 15, 16, 17 are Gamma or 0th harmonics */
+							/* 18, 19, 20 are 1st harmonics */
+							/* 21, 22, 23 are 2nd harmonics */
 							/* 24, 25, 26 etc. */
+							/* For isShTRC there is only one set of offsets & harmonics */
 	cow *points;			/* List of test points as dev->Lab */
 	int nodp;				/* Number of data points */
 } mxopt;
@@ -381,10 +401,10 @@ typedef struct {
 /* Per chanel function being optimised */
 static void mxmfunc1(mxopt *p, int j, double *v, double *out, double *in) {
 	double vv, g;
-	int ps = 3;		/* Parameter spacing */
-
-	vv = *in;
+	int ps = 3;				/* Parameter spacing */
 
+	vv = *in * p->dscale;
+	
 	if (p->isShTRC) {
 		j = 0;
 		ps = 1;				/* Only one channel */
@@ -411,15 +431,32 @@ static void mxmfunc1(mxopt *p, int j, double *v, double *out, double *in) {
 	} else {		/* Add extra shaper parameters */
 		int ord;
 
-		/* Apply gamma as order 0 */
-		g = v[9 + ps + j];
-		if (g <= 0.0)
-			vv = 1.0;
-		else {
-			if (vv >= 0.0) {
-				vv = pow(vv, g);
+		if (p->shape0gam) {
+
+			/* Apply input offset */
+			g = v[9 + j];		/* Offset value */
+	
+			if (g >= 1.0) {
+				vv = 1.0;
 			} else {
-				vv = -pow(-vv, g);
+				vv = g + ((1.0 - g) * vv);
+			}
+
+			/* Apply gamma as order 0 */
+			g = v[9 + 2 * ps + j];
+			if (g <= 0.0)
+				vv = 1.0;
+			else {
+				/* Power with straight line at small values */
+				if (vv >= XSHAPE_GAMTHR) {
+					vv = pow(vv, g);
+				} else {
+					double slope, oth;
+	
+					oth = pow(XSHAPE_GAMTHR, g);				/* Output at input threshold */
+					slope = g * pow(XSHAPE_GAMTHR, g - 1.0);	/* Slope at input threshold */
+					vv = oth + (vv - XSHAPE_GAMTHR) * slope;	/* Straight line */
+				}
 			}
 		}
 
@@ -431,10 +468,15 @@ static void mxmfunc1(mxopt *p, int j, double *v, double *out, double *in) {
 		/*  can't be non-monotonic. The control parameter has been */
 		/*  altered to have a range from -oo to +oo rather than 0.0 to 1.0 */
 		/*  so that the search space is less non-linear. ] */
-		for (ord = 1; ord < p->norders; ord++) {
+		if (p->shape0gam)
+			ord = 1;
+		else
+			ord = 0;
+		for (; ord < p->norders; ord++)
+		{
 			int nsec;			/* Number of sections */
 			double sec;			/* Section */
-			g = v[9 + ps + ord * ps + j];	/* Parameter */
+			g = v[9 + 2 * ps + ord * ps + j];	/* Parameter */
 	
 			nsec = ord + 1;		/* Increase sections for each order */
 	
@@ -453,19 +495,17 @@ static void mxmfunc1(mxopt *p, int j, double *v, double *out, double *in) {
 			vv /= (double)nsec;
 		}
 
-		/* Apply offset */
-		g = v[9 + j];		/* Offset value */
-
-		if (g >= 1.0) {
-			vv = 1.0;
-		} else if (g > 0.0) {	/* g > 0 && g < 1.0 */
-			vv = g + ((1.0 - g) * vv);
+		/* (For extrapolation it helps to pin 0 & 1) */
+		if (p->shape0gam) {
+			/* Apply output offset */
+			g = v[9 + 1 * ps + j];		/* Offset value */
+	
+			if (g >= 1.0) {
+				vv = 1.0;
+			} else if (g > 0.0) {
+				vv = g + ((1.0 - g) * vv);
+			}
 		}
-
-		if (vv < 0.0)
-			vv = 0.0;
-		else if (vv > 1.0)
-			vv = 1.0;
 	}
 
 	*out = vv;
@@ -499,41 +539,72 @@ double *v			/* Pointer to parameters */
 	}
 
 	if (p->isShTRC) {
-		/* Offset value */
+		/* Input offset value */
+		if (p->shape0gam)
+			w = XSHAPE_OFFG;
+		else
+			w = XSHAPE_OFFS;
+
 		tt = v[9];
 		tt *= tt;
-		w = XSHAPE_BASE;
+		tparam += w * tt;
+
+		/* Output offset value */
+		tt = v[10];
+		tt *= tt;
 		tparam += w * tt;
 
 		/* Shaper values */
 		for (f = 0; f < p->norders; f++) {
-			tt = v[12 + f];
+			tt = v[11 + f];
 			tt *= tt;
-			if (f <= 1)	/* First or second curves */
-				w = XSHAPE_BASE;
-			else
-				w = XSHAPE_BASE + (f-1) * XSHAPE_HBASE;
+			/* Weigh to suppress ripples */
+			if (f <= 1) {
+				w = XSHAPE_HW01;
+			} else if (f <= XSHAPE_HBREAK) {
+				double bl = (f - 1.0)/(XSHAPE_HBREAK - 1.0);
+				w = (1.0 - bl) * XSHAPE_HW01 + bl * XSHAPE_HWBR * p->smooth;
+			} else {
+				w = XSHAPE_HWBR + (f-XSHAPE_HBREAK) * XSHAPE_HWINC;
+				w *= p->smooth;
+			}
 			tparam += w * tt;
 		}
 		return XSHAPE_MAG * tparam;
 	}
 
-	/* Offset value */
+	/* Input ffset value */
+	if (p->shape0gam)
+		w = XSHAPE_OFFG;
+	else
+		w = XSHAPE_OFFS;
+
 	for (g = 0; g < 3; g++) {
 		tt = v[9 + g];
 		tt *= tt;
-		w = XSHAPE_BASE;
+		tparam += w * tt;
+	}
+	/* Output ffset value */
+	for (g = 0; g < 3; g++) {
+		tt = v[12 + g];
+		tt *= tt;
 		tparam += w * tt;
 	}
 	/* Shaper values */
 	for (f = 0; f < p->norders; f++) {
+		/* Weigh to suppress ripples */
+		if (f <= 1) {
+			w = XSHAPE_HW01;
+		} else if (f <= XSHAPE_HBREAK) {
+			double bl = (f - 1.0)/(XSHAPE_HBREAK - 1.0);
+			w = (1.0 - bl) * XSHAPE_HW01 + bl * XSHAPE_HWBR * p->smooth;
+		} else {
+			w = XSHAPE_HWBR + (f-XSHAPE_HBREAK) * XSHAPE_HWINC * p->smooth;
+			w *= p->smooth;
+		}
 		for (g = 0; g < 3; g++) {
-			tt = v[12 + 3 * f + g];
+			tt = v[15 + 3 * f + g];
 			tt *= tt;
-			if (f <= 1)	/* First or second curves */
-				w = XSHAPE_BASE;
-			else
-				w = XSHAPE_BASE + (f-1) * XSHAPE_HBASE;
 			tparam += w * tt;
 		}
 	}
@@ -543,21 +614,19 @@ double *v			/* Pointer to parameters */
 /* Matrix optimisation function handed to powell() */
 static double mxoptfunc(void *edata, double *v) {
 	mxopt *p = (mxopt *)edata;
-	double rv = 0.0, smv;
+	double err = 0.0, rv = 0.0, smv;
 	double xyz[3], lab[3];
 	int i;
 
 	for (i = 0; i < p->nodp; i++) {
 
 		/* Apply our function */
-//printf("%f %f %f -> %f %f %f\n", p->points[i].p[0], p->points[i].p[1], p->points[i].p[2],
-//xyz[0], xyz[1], xyz[2]);
+//printf("%f %f %f -> %f %f %f\n", p->points[i].p[0], p->points[i].p[1], p->points[i].p[2], xyz[0], xyz[1], xyz[2]);
 		mxmfunc(p, v, xyz, p->points[i].p);
 	
 		/* Convert to Lab */
 		icmXYZ2Lab(&icmD50, lab, xyz);
-//printf("%f %f %f -> %f %f %f, target %f %f %f\n", p->points[i].p[0], p->points[i].p[1], p->points[i].p[2],
-//lab[0], lab[1], lab[2], p->points[i].v[0], p->points[i].v[1], p->points[i].v[2]);
+//printf("%f %f %f -> %f %f %f, target %f %f %f\n", p->points[i].p[0], p->points[i].p[1], p->points[i].p[2], lab[0], lab[1], lab[2], p->points[i].v[0], p->points[i].v[1], p->points[i].v[2]);
 	
 		/* Accumulate total delta E squared */
 #ifdef USE_CIE94_DE
@@ -575,6 +644,32 @@ static double mxoptfunc(void *edata, double *v) {
 	smv = xshapmag(p, v);
 	rv += smv;
 
+	/* Penalize if we have -ve primaries */
+	if (p->clipbw) {
+		double tp[3];
+
+		tp[0] = tp[1] = tp[2] = 1.0;
+		mxmfunc(p, v, xyz, tp);
+		if ((xyz[1] - 1.0) > err)
+			err = xyz[1] - 1.0;
+
+		tp[0] = tp[1] = tp[2] = 0.0;
+		mxmfunc(p, v, xyz, tp);
+		for (i = 0; i < 3; i++) {
+			if (-xyz[i] > err)
+				err = -v[i];
+		}
+	}
+
+	/* Penalize if we have -ve primaries */
+	if (p->clipprims) {
+		for (i = 0; i < 9; i++) {
+			if (-v[i] > err)
+				err = -v[i];
+		}
+	}
+	rv += err * 1000.0;
+
 #ifdef DEBUG
 printf("~9(%f)mxoptfunc returning %f\n",smv,rv);
 #endif
@@ -595,6 +690,271 @@ static void mxprogfunc(void *pdata, int perc) {
 }
 
 
+/* Setup and then return the optimized matrix fit in the mxopt structure. */
+/* Return 0 on sucess, error code on failure. */
+static int
+createMatrix(
+char *err,			/* Return error message */
+mxopt *os,			/* Optimisation information */
+int verb,			/* NZ if verbose */
+int nodp,			/* Number of points */
+cow *ipoints,		/* Array of input points in XYZ space */
+int isLab,			/* nz if data points are Lab */
+int quality,		/* Quality metric, 0..3 (-1 == 2 orders only) */
+int isLinear,		/* NZ if pure linear, gamma = 1.0 */
+int isGamma,		/* NZ if gamma rather than shaper */
+int isShTRC,		/* NZ if shared TRCs */
+int shape0gam,      /* NZ if zero'th order shaper should be gamma function */
+int clipbw,			/* Prevent white > 1 and -ve black */
+int clipprims,		/* Prevent primaries going -ve */
+double smooth,		/* Smoothing factor (nominal 1.0) */
+double scale		/* Scale device values */
+) {
+	int inputChan = 3;			/* Must be RGB like */
+	int outputChan = 3;			/* Must be the PCS */
+	int rsplflags = 0;			/* Flags for scattered data rspl */
+	int e, f, i, j;
+	int maxits = 200;			/* Optimisation stop params */
+	double stopon = 0.01;		/* Absolute delta E change to stop on */
+	cow *points;				/* Lab copy of ipoints */
+	double rerr;
+
+#ifdef DEBUG_PLOT
+	#define	XRES 100
+	double xx[XRES];
+	double y1[XRES];
+#endif /* DEBUG_PLOT */
+
+	if (verb)
+		rsplflags |= RSPL_VERBOSE;
+
+	/* Allocate the array passed to fit_rspl() */
+	if ((points = (cow *)malloc(sizeof(cow) * nodp)) == NULL) {
+		if (err != NULL)
+			sprintf(err,"Allocation of scattered coordinate array failed");
+		return 2;
+	}
+
+	/* Setup points ready for optimisation */
+	for (i = 0; i < nodp; i++) {
+		for (e = 0; e < inputChan; e++)
+			points[i].p[e] = ipoints[i].p[e];
+		
+		for (f = 0; f < outputChan; f++)
+			points[i].v[f] = ipoints[i].v[f];
+
+		points[i].w = ipoints[i].w;
+
+		/* Make sure its Lab for delta E calculation */
+		if (isLab == 0)
+			icmXYZ2Lab(&icmD50, points[i].v, points[i].v);
+	}
+
+	/* Setup for optimising run */
+	if (verb != 0)
+		os->verb = verb;
+	else
+		os->verb = 0;
+	os->points = points;
+	os->nodp   = nodp;
+	os->isShTRC = 0;
+	os->shape0gam = shape0gam;
+	os->smooth = smooth;
+	os->clipbw = clipbw;
+	os->clipprims = clipprims;
+	os->dscale = scale;
+
+	/* Set quality/effort  factors */
+	if (quality >= 3) {			/* Ultra high */
+		os->norders = 20;
+		maxits = 10000;
+		stopon = 5e-7;
+	} else if (quality == 2) {	/* High */
+		os->norders = 12;
+		maxits = 4000;
+		stopon = 5e-6;
+	} else if (quality == 1) {	/* Medium */
+		os->norders = 8;
+		maxits = 2000;
+		stopon = 5e-5;
+	} else if (quality == 0) {  /* Low */
+		os->norders = 4;
+		maxits = 1000;
+		stopon = 0.0005;
+	} else {					/* Ultra Low */
+		os->norders = 2;
+		maxits = 1000;
+		stopon = 0.0005;
+	}
+	if (os->norders > MXNORDERS)
+		os->norders = MXNORDERS;
+	
+	/* Set initial optimisation values */
+	os->v[0] = 0.4;  os->v[1] = 0.4;  os->v[2] = 0.2;		/* Matrix */
+	os->v[3] = 0.2;  os->v[4] = 0.8;  os->v[5] = 0.1;
+	os->v[6] = 0.02; os->v[7] = 0.15; os->v[8] = 1.3;
+
+	if (isLinear) {				/* Linear */
+		os->isLinear = 1;
+		os->isGamma = 1;
+		os->optdim = 9;
+		os->v[9] = os->v[10] = os->v[11] = 1.0;					/* Linear */ 
+	} else if (isGamma) {		/* Just gamma curve */
+		os->isLinear = 0;
+		os->isGamma = 1;
+		os->optdim = 12;
+		os->v[9] = os->v[10] = os->v[11] = 2.4;					/* Gamma */ 
+	} else {		/* Creating input curves */
+		os->isLinear = 0;
+		os->isGamma = 0;
+		os->optdim = 9 + 6 + 3 * os->norders;		/* Matrix, offset + orders */
+		os->v[9] = os->v[10] = os->v[11] = 0.0;		/* Input offset */
+		os->v[12] = os->v[13] = os->v[14] = 0.0;	/* Output offset */
+		if (shape0gam)
+			os->v[15] = os->v[16] = os->v[17] = 2.0; 	/* Gamma */
+		else
+			os->v[15] = os->v[16] = os->v[17] = 0.0; 	/* 0th Harmonic */
+		for (i = 18; i < os->optdim; i++)
+			os->v[i] = 0.0; 						/* Higher orders */
+	}
+
+	/* Set search area to starting values */
+	for (j = 0; j < os->optdim; j++)
+		os->sa[j] = 0.2;					/* Matrix, Gamma, Offsets, harmonics */
+
+	if (isShTRC) {							/* Adjust things for shared */
+		os->isShTRC = 1;
+
+		if (os->optdim > 9) {
+			/* Pack red paramenters down */
+			for (i = 9; i < os->optdim; i++) {
+				os->v[i] = os->v[(i - 9) * 3 + 9];
+				os->sa[i] = os->sa[(i - 9) * 3 + 9];
+			}
+			os->optdim = ((os->optdim - 9)/3) + 9;
+		}
+	}
+
+	if (os->verb) {
+		if (os->isLinear)
+			printf("Creating matrix...\n"); 
+		else
+			printf("Creating matrix and curves...\n"); 
+	}
+
+	if (powell(&rerr, os->optdim, os->v, os->sa, stopon, maxits,
+	           mxoptfunc, (void *)os, mxprogfunc, (void *)&os) != 0)
+		warning("Powell failed to converge, residual error = %f",rerr);
+
+	if (os->clipprims) { /* Clip -ve primaries */
+		for (i = 0; i < 9; i++) {
+			if (os->v[i] < 0.0)
+				os->v[i] = 0.0;
+		}
+	}
+
+#ifndef NEVER
+	if (os->verb) {
+		printf("Matrix = %f %f %f\n",os->v[0], os->v[1], os->v[2]);
+		printf("         %f %f %f\n",os->v[3], os->v[4], os->v[5]);
+		printf("         %f %f %f\n",os->v[6], os->v[7], os->v[8]);
+		if (!isLinear) {		/* Creating input curves */
+			if (isGamma) {		/* Creating input curves */
+				if (isShTRC) 
+					printf("Gamma = %f\n",os->v[9]);
+				else
+					printf("Gamma = %f %f %f\n",os->v[9], os->v[10], os->v[11]);
+			} else {		/* Creating input curves */
+				if (isShTRC) { 
+					printf("Input offset  = %f\n",os->v[9]);
+					printf("Output offset = %f\n",os->v[10]);
+				} else {
+					printf("Input offset  = %f %f %f\n",os->v[9], os->v[10], os->v[11]);
+					printf("Output offset = %f %f %f\n",os->v[12], os->v[13], os->v[14]);
+				}
+				for (j = 0; j < os->norders; j++) {
+					if (isShTRC) { 
+						if (shape0gam)
+							printf("%d gamma = %f\n",j, os->v[11 + j]);
+						else
+							printf("%d harmonics = %f\n",j, os->v[11 + j]);
+					} else {
+						if (shape0gam)
+							printf("%d gamma = %f %f %f\n",j, os->v[15 + j * 3],
+							                    os->v[16 + j * 3], os->v[17 + j * 3]);
+						else
+							printf("%d harmonics = %f %f %f\n",j, os->v[15 + j * 3],
+							                    os->v[16 + j * 3], os->v[17 + j * 3]);
+					}
+				}
+			}
+		}
+	}
+#endif /* NEVER */
+
+	/* Free the coordinate lists */
+	free(points);
+
+	return 0;
+}
+
+static void icxMM_lookup(icxMatrixModel *p, double *out, double *in) {
+	mxopt *os = (mxopt *)p->imp;
+
+	mxmfunc(os, os->v, out, in);
+
+	if (p->isLab)
+		icmXYZ2Lab(&icmD50, out, out);
+}
+
+static void icxMM_del(icxMatrixModel *p) {
+	free(p->imp);
+	free(p);
+}
+
+/* Create a matrix model of a set of points, and return an object to lookup */
+/* points from the model. Return NULL on error. */
+icxMatrixModel *new_MatrixModel(
+int verb,			/* NZ if verbose */
+int nodp,			/* Number of points */
+cow *ipoints,		/* Array of input points in XYZ space */
+int isLab,			/* nz if data points are Lab */
+int quality,		/* Quality metric, 0..3  (-1 == 2 orders only) */
+int isLinear,		/* NZ if pure linear, gamma = 1.0 */
+int isGamma,		/* NZ if gamma rather than shaper */
+int isShTRC,		/* NZ if shared TRCs */
+int shape0gam,      /* NZ if zero'th order shaper should be gamma function */
+int clipbw,			/* Prevent white > 1 and -ve black */
+int clipprims,		/* Prevent primaries going -ve */
+double smooth,		/* Smoothing factor (nominal 1.0) */
+double scale		/* Scale device values */
+) {
+	icxMatrixModel *p;
+
+	if ((p = (icxMatrixModel *) calloc(1,sizeof(icxMatrixModel))) == NULL)
+		return NULL;
+
+	p->lookup = icxMM_lookup;
+	p->del = icxMM_del;
+
+	if ((p->imp = (void *) calloc(1,sizeof(mxopt))) == NULL) {
+		free(p);
+		return NULL;
+	}
+
+	if (createMatrix(NULL, (mxopt *)p->imp, verb, nodp, ipoints, isLab, quality,
+	                 isLinear, isGamma, isShTRC, shape0gam,
+		             clipbw, clipprims, smooth, scale) != 0) {
+		free(p->imp);
+		free(p);
+		return NULL;
+	}
+
+	p->isLab = isLab;
+
+	return p;
+}
+
 /* Create icxLuMatrix and undelying tone reproduction curves and */
 /* colorant tags, initialised from the icc, and then overwritten */
 /* by a conversion created from the supplied scattered data points. */
@@ -629,7 +989,6 @@ int                quality			/* Quality metric, 0..3 */
 	int e, f, i, j;
 	int maxits = 200;					/* Optimisation stop params */
 	double stopon = 0.01;				/* Absolute delta E change to stop on */
-	cow *points;		/* Copy of ipoints */
 	mxopt os;			/* Optimisation information */
 	double rerr;
 
@@ -712,137 +1071,17 @@ int                quality			/* Quality metric, 0..3 */
 
 	/* ------------------------------- */
 
-	/* Allocate the array passed to fit_rspl() */
-	if ((points = (cow *)malloc(sizeof(cow) * nodp)) == NULL) {
-		p->pp->errc = 2;
-		sprintf(p->pp->err,"Allocation of scattered coordinate array failed");
+	/* (Use a gamma curve as 0th order shape) */
+	if (p->pp->errc = createMatrix(p->pp->err, &os, flags & ICX_VERBOSE ? 1 : 0,  
+	                               nodp, ipoints, 0, quality,
+	                               isLinear, isGamma, isShTRC, 1,
+		                           flags & ICX_CLIP_WB ? 1 : 0,
+		                           flags & ICX_CLIP_PRIMS ? 1 : 0,
+		                           1.0, 1.0) != 0) {   
 		p->del((icxLuBase *)p);
 		return NULL;
 	}
 
-	/* Setup points ready for optimisation */
-	for (i = 0; i < nodp; i++) {
-		for (e = 0; e < inputChan; e++)
-			points[i].p[e] = ipoints[i].p[e];
-		
-		for (f = 0; f < outputChan; f++)
-			points[i].v[f] = ipoints[i].v[f];
-
-		points[i].w = ipoints[i].w;
-
-		/* Make sure its Lab for delta E calculation */
-		icmXYZ2Lab(&icmD50, points[i].v, points[i].v);
-	}
-
-	/* Setup for optimising run */
-	if (flags & ICX_VERBOSE)
-		os.verb = 1;
-	else
-		os.verb = 0;
-	os.points = points;
-	os.nodp   = nodp;
-	os.isShTRC = 0;
-
-	/* Set quality/effort  factors */
-	if (quality >= 3) {			/* Ultra high */
-		os.norders = 20;
-		maxits = 5000;
-		stopon = 0.000001;
-	} else if (quality == 2) {	/* High */
-		os.norders = 15;
-		maxits = 2000;
-		stopon = 0.00001;
-	} else if (quality == 1) {	/* Medium */
-		os.norders = 10;
-		maxits = 1000;
-		stopon = 0.0001;
-	} else {					/* Low */
-		os.norders = 5;
-		maxits = 500;
-		stopon = 0.001;
-	}
-	if (os.norders > MXNORDERS)
-		os.norders = MXNORDERS;
-	
-	/* Set initial optimisation values */
-	os.v[0] = 0.4;  os.v[1] = 0.4;  os.v[2] = 0.2;		/* Matrix */
-	os.v[3] = 0.2;  os.v[4] = 0.8;  os.v[5] = 0.1;
-	os.v[6] = 0.02; os.v[7] = 0.15; os.v[8] = 1.3;
-
-	if (isLinear) {		/* Just gamma curve */
-		os.isLinear = 1;
-		os.isGamma = 1;
-		os.optdim = 9;
-		os.v[9] = os.v[10] = os.v[11] = 1.0;					/* Linear */ 
-	} else if (isGamma) {		/* Just gamma curve */
-		os.isLinear = 0;
-		os.isGamma = 1;
-		os.optdim = 12;
-		os.v[9] = os.v[10] = os.v[11] = 2.4;					/* Gamma */ 
-	} else {		/* Creating input curves */
-		os.isLinear = 0;
-		os.isGamma = 0;
-		os.optdim = 12 + 3 * os.norders;
-		os.v[9] = os.v[10] = os.v[11] = 0.0;	/* Offset */
-		os.v[12] = os.v[13] = os.v[14] = 2.0; 	/* 0th Harmonic */
-		for (i = 15; i < os.optdim; i++)
-			os.v[i] = 0.0; 						/* Higher orders */
-	}
-
-	/* Set search area to starting values */
-	for (j = 0; j < os.optdim; j++)
-		os.sa[j] = 0.2;					/* Matrix, Gamma, Offsets, harmonics */
-
-	if (isShTRC) {						/* Adjust things for shared */
-		os.isShTRC = 1;
-
-		if (os.optdim > 9) {
-			/* Pack red paramenters down */
-			for (i = 9; i < os.optdim; i++) {
-				os.v[i] = os.v[(i - 9) * 3 + 9];
-				os.sa[i] = os.sa[(i - 9) * 3 + 9];
-			}
-			os.optdim = ((os.optdim - 9)/3) + 9;
-		}
-	}
-
-	if (os.verb) {
-		if (os.isLinear)
-			printf("Creating matrix...\n"); 
-		else
-			printf("Creating matrix and curves...\n"); 
-	}
-
-	if (powell(&rerr, os.optdim, os.v, os.sa, stopon, maxits,
-	           mxoptfunc, (void *)&os, mxprogfunc, (void *)&os) != 0)
-		warning("Powell failed to converge, residual error = %f",rerr);
-
-#ifdef NEVER
-printf("Matrix = %f %f %f\n",os.v[0], os.v[1], os.v[2]);
-printf("         %f %f %f\n",os.v[3], os.v[4], os.v[5]);
-printf("         %f %f %f\n",os.v[6], os.v[7], os.v[8]);
-if (!isLinear) {		/* Creating input curves */
-	if (isGamma) {		/* Creating input curves */
-		if (isShTRC) 
-			printf("Gamma = %f\n",os.v[9]);
-		else
-			printf("Gamma = %f %f %f\n",os.v[9], os.v[10], os.v[11]);
-	} else {		/* Creating input curves */
-		if (isShTRC) 
-			printf("Offset = %f\n",os.v[9]);
-		else
-			printf("Offset = %f %f %f\n",os.v[9], os.v[10], os.v[11]);
-		for (j = 0; j < os.norders; j++) {
-			if (isShTRC) 
-				printf("%d harmonics = %f\n",j, os.v[10 + j]);
-			else
-				printf("%d harmonics = %f %f %f\n",j, os.v[12 + j * 3], os.v[13 + j * 3],
-				                                           os.v[14 + j * 3]);
-		}
-	}
-}
-#endif /* NEVER */
-
 	/* Deal with white/black points */
 	if (flags & (ICX_SET_WHITE | ICX_SET_BLACK)) {
 		double wp[3];	/* Absolute White point in XYZ */
@@ -857,8 +1096,8 @@ if (!isLinear) {		/* Creating input curves */
 		/* Figure out the device values for white */
 		if (h->deviceClass == icSigInputClass) {
 			double dwhite[MXDI], dblack[MXDI];	/* Device white and black values */
-			double Lmax = -1e60;
-			double Lmin = 1e60;
+			double Ymax = -1e60;
+			double Ymin = 1e60;
 
 			/* We assume that the input target is well behaved, */
 			/* and that it includes a white and black point patch, */
@@ -874,23 +1113,23 @@ if (!isLinear) {		/* Creating input curves */
 
 			/* Discover the white and black points */
 			for (i = 0; i < nodp; i++) {
-				if (points[i].v[0] > Lmax) {
-					Lmax = 
-					wp[0] = points[i].v[0];
-					wp[1] = points[i].v[1];
-					wp[2] = points[i].v[2];
-					dwhite[0] = points[i].p[0];
-					dwhite[1] = points[i].p[1];
-					dwhite[2] = points[i].p[2];
+				if (ipoints[i].v[1] > Ymax) {
+					Ymax = 
+					wp[0] = ipoints[i].v[0];
+					wp[1] = ipoints[i].v[1];
+					wp[2] = ipoints[i].v[2];
+					dwhite[0] = ipoints[i].p[0];
+					dwhite[1] = ipoints[i].p[1];
+					dwhite[2] = ipoints[i].p[2];
 				}
-				if (points[i].v[0] < Lmin) {
-					Lmin = 
-					bp[0] = points[i].v[0];
-					bp[1] = points[i].v[1];
-					bp[2] = points[i].v[2];
-					dblack[0] = points[i].p[0];
-					dblack[1] = points[i].p[1];
-					dblack[2] = points[i].p[2];
+				if (ipoints[i].v[1] < Ymin) {
+					Ymin = 
+					bp[0] = ipoints[i].v[0];
+					bp[1] = ipoints[i].v[1];
+					bp[2] = ipoints[i].v[2];
+					dblack[0] = ipoints[i].p[0];
+					dblack[1] = ipoints[i].p[1];
+					dblack[2] = ipoints[i].p[2];
 				}
 			}
 
@@ -898,6 +1137,28 @@ if (!isLinear) {		/* Creating input curves */
 			mxmfunc(&os, os.v, wp, dwhite);
 			mxmfunc(&os, os.v, bp, dblack);
 
+			if (flags & ICX_CLIP_WB) {
+				if (wp[2] > 1.0) {
+					if (flags & ICX_VERBOSE)
+						printf("Clipping white point from XYZ %f %f %f",wp[0],wp[1],wp[2]);
+					icmScale3(wp, wp, 1.0/wp[2]);
+					if (flags & ICX_VERBOSE)
+						printf(" to XYZ %f %f %f\n",wp[0],wp[1],wp[2]);
+				}
+				if (bp[0] < 0.0 || bp[1] < 0.0 || bp[1] < 0.0) {
+					if (flags & ICX_VERBOSE)
+						printf("Clipping black point from XYZ %f %f %f",bp[0],bp[1],bp[2]);
+					if (bp[0] < 0.0)
+						bp[0] = 0.0;
+					if (bp[1] < 0.0)
+						bp[1] = 0.0;
+					if (bp[2] < 0.0)
+						bp[2] = 0.0;
+					if (flags & ICX_VERBOSE)
+						printf(" to XYZ %f %f %f\n",bp[0],bp[1],bp[2]);
+				}
+			}
+
 			/* If we were given an input white point scale factor, apply it */
 			if (wpscale >= 0.0) {
 				wp[0] *= wpscale;
@@ -964,6 +1225,22 @@ if (!isLinear) {		/* Creating input curves */
 					break;
 				}
 			}
+
+			if (flags & ICX_CLIP_WB) {
+				/* Don't clip white, as it will be scaled to 1.0 anyway */
+				if (bp[0] < 0.0 || bp[1] < 0.0 || bp[1] < 0.0) {
+					if (flags & ICX_VERBOSE)
+						printf("Clipping black point from XYZ %f %f %f",bp[0],bp[1],bp[2]);
+					if (bp[0] < 0.0)
+						bp[0] = 0.0;
+					if (bp[1] < 0.0)
+						bp[1] = 0.0;
+					if (bp[2] < 0.0)
+						bp[2] = 0.0;
+					if (flags & ICX_VERBOSE)
+						printf(" to XYZ %f %f %f\n",bp[0],bp[1],bp[2]);
+				}
+			}
 		}
 
 		/* If this is a display, adjust the white point to be */
@@ -1090,6 +1367,14 @@ if (!isLinear) {		/* Creating input curves */
 			os.v[0] = mat[0][0]; os.v[1] = mat[0][1]; os.v[2] = mat[0][2];
 			os.v[3] = mat[1][0]; os.v[4] = mat[1][1]; os.v[5] = mat[1][2];
 			os.v[6] = mat[2][0]; os.v[7] = mat[2][1]; os.v[8] = mat[2][2];
+
+			/* Hmm. Ideally this should be within the matrix optimzation.. */
+			if (flags & ICX_CLIP_PRIMS) {
+				for (i = 0; i < 9; i++) {
+					if (os.v[i] < 0.0)
+						os.v[i] = 0.0;
+				}
+			}
 			if (flags & ICX_VERBOSE) {
 				printf("After white point adjust:\n");
 				printf("Matrix = %f %f %f\n",os.v[0], os.v[1], os.v[2]);
@@ -1119,6 +1404,10 @@ if (!isLinear) {		/* Creating input curves */
 	
 				mxmfunc1(&os, j, os.v, &rgb[j], &in);
 
+				if (rgb[j] < 0.0)
+					rgb[j] = 0.0;
+				else if (rgb[j] > 1.0)
+					rgb[j] = 1.0;
 			}
 			wor->data[ui] = rgb[0];	/* Curve values 0.0 - 1.0 */
 			if (!isShTRC) {
@@ -1133,6 +1422,10 @@ if (!isLinear) {		/* Creating input curves */
 				double x, y;
 				xx[i] = x = i/(double)(XRES-1);
 				mxmfunc1(&os, j, os.v, &y, &x);
+				if (y < 0.0)
+					y = 0.0;
+				else if (y > 1.0)
+					y = 1.0;
 				y1[i] = y;
 			}
 			do_plot(xx,y1,NULL,NULL,XRES);
@@ -1165,9 +1458,6 @@ if (!isLinear) {		/* Creating input curves */
 		/* Load into pmlu matrix and inverse ??? */
 	}
 
-	/* Free the coordinate lists */
-	free(points);
-
 	if (flags & ICX_VERBOSE)
 		printf("Profile done\n");
 
